
/**
* MACS is an Anticipatory Learning Classifier System written in Java.
* 
* Copyright (C) Pierre Grard and Olivier Sigaud with the occasional help
* of Fabien Flacher.
* Contact Olivier.Sigaud@lip6.fr for any queries, or write to
* 
* Olivier Sigaud
* AnimatLab
* Laboratoire d'Informatique de Paris 6
* 8, rue du Capitaine Scott,
* 75015 PARIS, FRANCE
* 
* This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
* 
* More informations about the GNU General Public License can be found
* at the following web site:
* http://www.opensource.org/licenses/gpl-license.php
**/

package Classifier;

import Utils.StaticRandom;

/**
* This class is used to represent the action part of a classifier
* @author <a href="mailto:Olivier.Sigaud@lip6.fr"> Olivier Sigaud</a> and <a href="mailto:gerpy@free.fr"> Pierre Grard</a>.
* 
**/

public final class Action extends Message 
{
   
   /**
    * @param action
    */
   public Action(Action action) 
   {
	super((Message)action);
	specialization = 1;
	isSpecializationUpdated = true;    
   }
   
   /**
    * @param strInit
    * @param form
    */
   public Action(String strInit, String form) 
   {
	this(form);
	setValue (strInit);    
   }
   
   /**
    * @param form
    */
   public Action(String form) 
   {
	byte nbValuesArray[] = new byte[computeNbTokens(form)];
	length = computeNbTokens(form, nbValuesArray);

	tokenArray = new  Token[length];
	nbSpecValues = new byte[length];

	for (byte i=0; i<length; i++)
	    {
		tokenArray[i] = TokenRepository.dontCareToken;
		setNbSpecValues(i, nbValuesArray[i]);
	    }    
   }
   
   Action() 
   {
	specialization = 1;
	isSpecializationUpdated = true;    
   }
   
   /**
    * @return double
    */
   public final double getSpecialization() 
   {
	return 1.0;    
   }
   
   /**
    * @return boolean
    */
   public final boolean isGeneral() 
   {
	return (false);    
   }
   
   /**
    * @param message
    * @return boolean
    */
   public final boolean isMoreGeneralThan(Message message) 
   {
	return false;    
   }
}
