
/**
* MACS is an Anticipatory Learning Classifier System written in Java.
* 
* Copyright (C) Pierre Grard and Olivier Sigaud with the occasional help
* of Fabien Flacher.
* Contact Olivier.Sigaud@lip6.fr for any queries, or write to
* 
* Olivier Sigaud
* AnimatLab
* Laboratoire d'Informatique de Paris 6
* 8, rue du Capitaine Scott,
* 75015 PARIS, FRANCE
* 
* This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
* 
* More informations about the GNU General Public License can be found
* at the following web site:
* http://www.opensource.org/licenses/gpl-license.php
**/

package Classifier;

import java.util.Vector;
import Utils.StaticRandom;

/**
* This class is used to represent the anticipation part of a MACS classifier.
* @author <a href="mailto:Olivier.Sigaud@lip6.fr"> Olivier Sigaud</a> and <a href="mailto:gerpy@free.fr"> Pierre Grard</a>.
* 
**/

public final class MACSAnticipation extends Perception implements Anticipation 
{
   
   /**
    * @param perception
    */
   public MACSAnticipation(MACSAnticipation perception) 
   {
	super((Perception)perception);    
   }
   
   /**
    * @param strInit
    * @param form
    */
   public MACSAnticipation(String strInit, String form) 
   {
	super(form);
	setValue(strInit);    
   }
   
   /**
    * Creation of a random anticipation
    * 
    * @param form
    * @param val
    */
   public MACSAnticipation(String form, boolean val) 
   {
	byte nbValuesArray[] = new byte[computeNbTokens(form)];
	length = computeNbTokens(form, nbValuesArray);

	tokenArray = new Token[length];
	nbSpecValues = new byte[length];

	for(byte i=0; i<length; i++)
	    {
		setNbSpecValues(i,(byte)2);
		if (StaticRandom.nextDouble()<0.5) tokenArray[i] = TokenRepository.getToken((byte)0);
		else tokenArray[i]= TokenRepository.getToken((byte)1);
	    }    
   }
   
   /**
    * @param form
    */
   public MACSAnticipation(String form) 
   {
	byte nbValuesArray[] = new byte[computeNbTokens(form)];
	length = computeNbTokens(form, nbValuesArray);

	tokenArray = new Token[length];
	nbSpecValues = new byte[length];

	for(byte i=0; i<length; i++)
	    {
		tokenArray[i] = TokenRepository.dontKnowToken;
		setNbSpecValues(i, nbValuesArray[i]);
	    }    
   }
   
   /**
    * @return byte
    */
   public final byte getIndexOfFirstSpecializedToken() 
   {
	for (byte i=0 ; i<length ; ++i)
	    if (!tokenArray[i].isDontKnow()) return i;
	return (byte)(-1);    
   }
   
   /**
    * @param i
    * @return boolean
    */
   public final boolean isSpecialized(byte i) 
   {
	return (!tokenArray[i].isDontKnow());    
   }
   
   /**
    * @param message
    * @return boolean
    */
   public final boolean matches(Message message) 
   {
	for (byte i=0 ; i<length ; ++i)
	    if ((isSpecialized(i)) && (!tokenArray[i].equals(message.tokenArray[i]))) return false;
	return true;    
   }
   
   /**
    * @param message
    * @return boolean
    */
   public final boolean isCompatibleWith(Anticipation message) 
   {
	for (byte i=0 ; i<length ; ++i)
	    if ((isSpecialized(i)) && (((MACSAnticipation)message).isSpecialized(i)) && (!tokenArray[i].equals(((MACSAnticipation)message).tokenArray[i]))) return false;
	return true;    
   }
}
