
/**
* MACS is an Anticipatory Learning Classifier System written in Java.
* 
* Copyright (C) Pierre Grard and Olivier Sigaud with the occasional help
* of Fabien Flacher.
* Contact Olivier.Sigaud@lip6.fr for any queries, or write to
* 
* Olivier Sigaud
* AnimatLab
* Laboratoire d'Informatique de Paris 6
* 8, rue du Capitaine Scott,
* 75015 PARIS, FRANCE
* 
* This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
* 
* More informations about the GNU General Public License can be found
* at the following web site:
* http://www.opensource.org/licenses/gpl-license.php
**/

package Classifier;

import java.util.Vector;
import Utils.StaticRandom;

/**
* This class is used to represent the perception part of classifiers.
* @author <a href="mailto:Olivier.Sigaud@lip6.fr"> Olivier Sigaud</a> and <a href="mailto:gerpy@free.fr"> Pierre Grard</a>.
* 
**/

public class Perception extends Message 
{
   
   /**
    * @param perception
    */
   public Perception(Perception perception) 
   {
	super((Message)perception);    
   }
   
   /**
    * @param strInit
    * @param form
    */
   public Perception(String strInit, String form) 
   {
	this(form);
	setValue(strInit);    
   }
   
   /**
    * Creation of a random anticipation
    * 
    * @param form
    * @param val
    */
   public Perception(String form, boolean val) 
   {
	byte nbValuesArray[] = new byte[computeNbTokens(form)];
	length = computeNbTokens(form, nbValuesArray);

	tokenArray = new Token[length];
	nbSpecValues = new byte[length];

	for(byte i=0; i<length; i++)
	    {
		setNbSpecValues(i,(byte)2);
		if (StaticRandom.nextDouble()<0.5) tokenArray[i] = TokenRepository.getToken((byte)0);
		else tokenArray[i]= TokenRepository.getToken((byte)1);
	    }    
   }
   
   /**
    * @param form
    */
   public Perception(String form) 
   {
	byte nbValuesArray[] = new byte[computeNbTokens(form)];
	length = computeNbTokens(form, nbValuesArray);

	tokenArray = new Token[length];
	nbSpecValues = new byte[length];

	for(byte i=0; i<length; i++)
	    {
		tokenArray[i] = TokenRepository.dontCareToken;
		setNbSpecValues(i, nbValuesArray[i]);
	    }    
   }
   
   public Perception() 
   {
    
   }
   
   /**
    * @param i
    * @return boolean
    */
   public boolean isSpecialized(byte i) 
   {
	return (!tokenArray[i].isDontCare());    
   }
   
   /**
    * @return byte
    */
   public byte getIndexOfFirstSpecializedToken() 
   {
	for (byte i=0 ; i<length ; ++i)
	    if (!tokenArray[i].isDontCare()) return i;
	return (byte)(-1);    
   }
   
   /**
    * @param perception
    * @return Perception
    */
   protected final Perception passThrough(Anticipation perception) 
   {
	Perception newPerception = new Perception(this);

	for(byte i=0 ; i<length ; i++)
	    newPerception.tokenArray[i] = tokenArray[i].passThrough(((Perception)perception).tokenArray[i]);	    

	return newPerception;    
   }
   
   /**
    * @param perception
    * @return Perception
    */
   protected final Perception diffWith(Perception perception) 
   {
	Perception newPerception = new Perception(this);

	for(byte i=0 ; i<length ; i++)
	    newPerception.tokenArray[i] = tokenArray[i].diffWith(perception.tokenArray[i]);	    

	return newPerception;    
   }
   
   /**
    * @param perception
    */
   public final void correctSoAsToMatch(Perception perception) 
   {
	for(byte i=0 ; i<length ; i++)
	    if (isSpecialized(i) && (getToken(i)!=perception.getToken(i)))
		{
		    setToken(i,TokenRepository.getToken(perception.getToken(i).getValue()));
		}    
   }
   
   /**
    * @param index
    * @return byte
    */
   public final byte getNbSpecValues(byte index) 
   {
	if (tokenArray[index].isDontCare())	return getNbSpecValues(index);
	else					return (byte)0;    
   }
}
