
/**
* MACS is an Anticipatory Learning Classifier System written in Java.
* 
* Copyright (C) Pierre Grard and Olivier Sigaud with the occasional help
* of Fabien Flacher.
* Contact Olivier.Sigaud@lip6.fr for any queries, or write to
* 
* Olivier Sigaud
* AnimatLab
* Laboratoire d'Informatique de Paris 6
* 8, rue du Capitaine Scott,
* 75015 PARIS, FRANCE
* 
* This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
* 
* More informations about the GNU General Public License can be found
* at the following web site:
* http://www.opensource.org/licenses/gpl-license.php
**/

package Classifier;

import java.lang.reflect.Array;
import Utils.StaticRandom;

/**
* This class is used to represent single units in messages.
* @author <a href="mailto:Olivier.Sigaud@lip6.fr"> Olivier Sigaud</a> and <a href="mailto:gerpy@free.fr"> Pierre Grard</a>.
* 
**/

public final class Token implements Comparable 
{
   private static final byte DONT_CARE = Byte.MAX_VALUE;
   private static final byte DONT_CHANGE = Byte.MAX_VALUE;
   
   private static final byte DONT_KNOW = Byte.MAX_VALUE;
   private byte value;
   
   /**
    * @param string
    */
   protected Token(String string) 
   {
	if (string.equals("#")) value = DONT_CARE;
	else if (string.equals("=")) value = DONT_CHANGE; 
	else if (string.equals("?")) value = DONT_KNOW;
	else   value = Byte.valueOf(string).byteValue();
   }
   
   /**
    * @return boolean
    */
   public final boolean isDontCare() 
   {
	return (this==TokenRepository.dontCareToken);    
   }
   
   /**
    * @return boolean
    */
   public final boolean isDontChange() 
   {
	return (this==TokenRepository.dontChangeToken);    
   }
   
   /**
    * @return boolean
    */
   public final boolean isDontKnow() 
   {
	return (this==TokenRepository.dontKnowToken);    
   }
   
   /**
    * @return byte
    */
   public final byte getValue() 
   {
	return value;    
   }
   
   /**
    * @param token
    * @return boolean
    */
   protected final boolean matches(Token token) 
   {
	return ((this==TokenRepository.dontCareToken) || (token==TokenRepository.dontCareToken) || (this == token));    
   }
   
   /**
    * @param token
    * @return boolean
    */
   protected final boolean isMoreGeneralThan(Token token) 
   {
        return ((this==TokenRepository.dontCareToken) && (token!=TokenRepository.dontCareToken));    
   }
   
   /**
    * @param token
    * @return boolean
    */
   protected final boolean isLessGeneralThan(Token token) 
   {
        return ((this!=TokenRepository.dontCareToken) && (token==TokenRepository.dontCareToken));    
   }
   
   /**
    * @param token
    * @return Token
    */
   protected final Token diffWith(Token token) 
   {
	if (this.equals(token)) return TokenRepository.dontChangeToken;
	else return this;    
   }
   
   /**
    * @param token
    * @return Token
    */
   protected final Token passThrough(Token token) 
   {
	if (token.isDontChange()) return this;
	else    return token;    
   }
   
   /**
    * @param token
    * @return Token
    */
   protected final Token buildForMutSpec(Token token) 
   {
	if (this.equals(token)) return TokenRepository.dontCareToken;
	else return token;    
   }
   
   /**
    * @return String
    */
   public final String toString() 
   {
	if (isDontCare())	    return ("#");
	else if (isDontChange())	    return ("=");
	else if (value==DONT_KNOW)	    return ("?");
	else   return (String.valueOf((int)value));    
   }
   
   /**
    * @param p
    * @return int
    */
   public final int compareTo(Object p) 
   {
	if (value>((Token)p).getValue()) return 1;
	if (value<((Token)p).getValue()) return -1;
	return 0;    
   }
}
