
/**
* MACS is an Anticipatory Learning Classifier System written in Java.
* 
* Copyright (C) Pierre Grard and Olivier Sigaud with the occasional help
* of Fabien Flacher.
* Contact Olivier.Sigaud@lip6.fr for any queries, or write to
* 
* Olivier Sigaud
* AnimatLab
* Laboratoire d'Informatique de Paris 6
* 8, rue du Capitaine Scott,
* 75015 PARIS, FRANCE
* 
* This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
* 
* More informations about the GNU General Public License can be found
* at the following web site:
* http://www.opensource.org/licenses/gpl-license.php
**/

package Classifier;

import java.lang.reflect.Array;
import Utils.StaticRandom;

/**
 * The TokenRepository class is used to store the set of all Tokens used by the LCS
 * All tokens are created once and for all so as to prevent 
 * the extensive use of "new".
 * They are static so that every class can access them easily
 * Any comparison between token is made just through their addresses
 * It would be nice if this class could be made local to the package Classifier
 * i.e. remove "public" in the class declaration
 * So far, it is not possible because init is called from outside...

* @author <a href="mailto:Olivier.Sigaud@lip6.fr"> Olivier Sigaud</a> and <a href="mailto:gerpy@free.fr"> Pierre Grard</a>.
* 
 */
public final class TokenRepository 
{
    public static boolean DEBUG = false;

    public static Token dontCareToken;
    public static Token dontChangeToken;
    public static Token dontKnowToken;
   private static boolean isInitialized = false;
   public static Token tokenValues[];
   
   /**
    * @param nbSpec
    */
   public static void init(byte nbSpec) 
   {
	if (isInitialized)
	    {
		if (DEBUG) System.out.println("Tentative de rinitialisation du TokenRepository...");
	    }
	else
	    {
		dontCareToken = new Token("#");
		dontChangeToken = new Token("=");
		dontKnowToken = new Token("?");
		tokenValues = new Token[nbSpec+1];
		for (byte i=0;i<=nbSpec;i++)
		    {
			tokenValues[i] = new Token(""+i);
		    }
		isInitialized = true;
	    }    
   }
   
   /**
    * @return Classifier.Token[]
    */
   public static final Token[] getSpecializedSet() 
   {
	return tokenValues;    
   }
   
   /**
    * @param string
    * @return Classifier.Token
    */
   public static final Token getToken(String string) 
   {
	if (string.equals("#")) return dontCareToken;
	else if (string.equals("=")) return dontChangeToken;
	else if (string.equals("?")) return dontKnowToken;
	else   return tokenValues[Byte.valueOf(string).byteValue()];    
   }
   
   /**
    * @param val
    * @return Classifier.Token
    */
   public static final Token getToken(byte val) 
   {
	return tokenValues[val];    
   }
}
