
/**
* MACS is an Anticipatory Learning Classifier System written in Java.
* 
* Copyright (C) Pierre Grard and Olivier Sigaud with the occasional help
* of Fabien Flacher.
* Contact Olivier.Sigaud@lip6.fr for any queries, or write to
* 
* Olivier Sigaud
* AnimatLab
* Laboratoire d'Informatique de Paris 6
* 8, rue du Capitaine Scott,
* 75015 PARIS, FRANCE
* 
* This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
* 
* More informations about the GNU General Public License can be found
* at the following web site:
* http://www.opensource.org/licenses/gpl-license.php
**/

package Environments;

import Utils.StaticRandom;
import Utils.Misc;
import java.util.Vector;

/**
 * This class is used to represent an agent into a Woods Environment.
* @author <a href="mailto:Olivier.Sigaud@lip6.fr"> Olivier Sigaud</a> and <a href="mailto:gerpy@free.fr"> Pierre Grard</a>.
 */

public class Agent implements Drawable 
{
   protected Coords current = new Coords ();
   
   /**
    * @param x
    * @param y
    */
   public void goTo(short x, short y) 
   {
	current.set(x,y);    
   }
   
   /**
    * @param c
    */
   public void goTo(Coords c) 
   {
	current.set(c.getXI(),c.getYI());    
   }
   
   /**
    * @param x
    * @param y
    * @return boolean
    */
   public boolean isIn(short x, short y) 
   {
	return (current.getXI()==x) && (current.getYI()==y);    
   }
   
   /**
    * @return int
    */
   public final int getXI() 
   {
	return current.getXI();    
   }
   
   /**
    * @return int
    */
   public final int getYI() 
   {
	return current.getYI();    
   }
   
   /**
    * @return Environments.Coords
    */
   public final Coords getCoords() 
   {
	return current;    
   }
   
   /**
    * @return double
    */
   public double getX() 
   {
	return (double)current.getXI();    
   }
   
   /**
    * @return double
    */
   public double getY() 
   {
	return (double)current.getYI();    
   }
   
   /**
    * @param action
    * @param xSize
    * @param ySize
    */
   public void move(byte action, short xSize, short ySize) 
   {
	switch (action)
	    {
	    case 0:
		current.setY((current.getYI()-1+ySize) % ySize);
		break;
	    case 1:
		current.set(((current.getXI()+1) % xSize),((current.getYI()-1+ySize) % ySize));
		break;
	    case 2:
		current.setX((current.getXI()+1) % xSize);
		break;
	    case 3:
		current.set(((current.getXI()+1) % xSize),((current.getYI()+1) % ySize));
		break;
	    case 4:
		current.setY((current.getYI()+1) % ySize);
		break;
	    case 5:
		current.set(((current.getXI()-1+xSize) % xSize),((current.getYI()+1) % ySize));
		break;
	    case 6:
		current.setX((current.getXI()-1+xSize) % xSize);
		break;
	    case 7:
		current.set(((current.getXI()-1+xSize) % xSize),((current.getYI()-1+ySize) % ySize));
		break;
	    default:
		System.out.println("Agent : action inconnnue");
	    }    
	//System.out.println("Agent move :" + action + " : " + toString());
   }
   
   /**
    * @return String
    */
   public final String toString() 
   {
	return new String("Agent positionn en :" + current);    
   }
}
/**
 *  end of Agent
 */
