
/**
* MACS is an Anticipatory Learning Classifier System written in Java.
* 
* Copyright (C) Pierre Grard and Olivier Sigaud with the occasional help
* of Fabien Flacher.
* Contact Olivier.Sigaud@lip6.fr for any queries, or write to
* 
* Olivier Sigaud
* AnimatLab
* Laboratoire d'Informatique de Paris 6
* 8, rue du Capitaine Scott,
* 75015 PARIS, FRANCE
* 
* This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
* 
* More informations about the GNU General Public License can be found
* at the following web site:
* http://www.opensource.org/licenses/gpl-license.php
**/

package Environments;

import Utils.StaticRandom;
import Utils.Misc;
import java.util.Vector;

/**
 * This class is used to represent a cell of a Woods Environment.
 * It is useful if you want to have a graphical display of the environment.
* @author <a href="mailto:Olivier.Sigaud@lip6.fr"> Olivier Sigaud</a> and <a href="mailto:gerpy@free.fr"> Pierre Grard</a>.
 */

public class Cell implements Drawable 
{
   private int content;
   protected Coords coords = new Coords ();
   private String perception;

   
   /**
    * @param c
    * @param x
    * @param y
    */
   public Cell(int c, int x, int y) 
   {
	content = c;
	coords = new Coords(x,y);
   }
   
   /**
    * @return double
    */
   public double getX() 
   {
	return (double)coords.getXI();    
   }
   
   /**
    * @param int la coordonnee
    */
   public void setY(int y) 
   {
	coords.setY(y);    
   }
   
   /**
    * @return double
    */
   public double getY() 
   {
	return (double)coords.getYI();    
   }
   
   /**
    * @return int
    */
   public int getContent() 
   {
	return content;    
   }
   
   /**
    * @return String the perception that can be seen from that cell
    */
   public String getPerception() 
   {
	return perception;
   }
   
   /**
    * @param p the perception that can be seen from that cell
    */
   public void setPerception(String p) 
   {
	perception = p;   
   } 
   /**
    * @return boolean
    */
   public boolean isFreeCell() 
   {
	return (content==0);    
   }
   
   /**
    * @return boolean
    */
   public boolean isWallCell() 
   {
	return (content==1);    
   }
   
   /**
    * @return boolean
    */
   public boolean isFoodCell() 
   {
	return (content==2);    
   }
   
   /**
    * @return boolean
    */
   public boolean isTrois() 
   {
	return (content==3);    
   }
   
   /**
    * @param b
    */
   public void setContent(int b) 
   {
	content=b;    
   }
   
   /**
    * @return String
    */
   public final String toString() 
   {
	return "" + content;
   }
}
