
/**
* MACS is an Anticipatory Learning Classifier System written in Java.
* 
* Copyright (C) Pierre Grard and Olivier Sigaud with the occasional help
* of Fabien Flacher.
* Contact Olivier.Sigaud@lip6.fr for any queries, or write to
* 
* Olivier Sigaud
* AnimatLab
* Laboratoire d'Informatique de Paris 6
* 8, rue du Capitaine Scott,
* 75015 PARIS, FRANCE
* 
* This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
* 
* More informations about the GNU General Public License can be found
* at the following web site:
* http://www.opensource.org/licenses/gpl-license.php
**/

package Environments;

/**
 * This class is used to represent the coordinates of an agent or a cell in a Woods Environment.
 * It is useful if you want to have a graphical display of the environment.
* @author <a href="mailto:Olivier.Sigaud@lip6.fr"> Olivier Sigaud</a> and <a href="mailto:gerpy@free.fr"> Pierre Grard</a>.
 */

public class Coords
{
   private int x;
   private int y;
   
   /**
    * @param source
    */
   public Coords(Coords source) 
   {
	x = source.x;
	y = source.y;    
   }
   
   /**
    * @param xinit
    * @param yinit
    */
   public Coords(int xinit, int yinit) 
   {
	x = xinit;
	y = yinit;    
   }
   
   public Coords() 
   {
    
   }
   
   /**
    * @param x1
    * @param y1
    */
   public final void set(int x1, int y1) 
   {
	x = x1;
	y = y1;    
   }
   
   /**
    * @param x1
    */
   public final void setX(int x1) 
   {
	x = x1;    
   }
   
   /**
    * @param y1
    */
   public final void setY(int y1) 
   {
	y = y1;    
   }
   
   /**
    * @return int
    */
   public final int getXI() 
   {
	return x;    
   }
   
   /**
    * @return int
    */
   public final int getYI() 
   {
	return y;    
   }
   
   /**
    * @return double
    */
   public final double getX() 
   {
	return (double)x;    
   }
   
   /**
    * @return double
    */
   public final double getY() 
   {
	return (double)y;    
   }
   
   /**
    * @param coord
    * @return boolean
    */
   public final boolean equals(Coords coord) 
   {
	return (x==coord.x) && (y==coord.y);    
   }
   
   /**
    * @return String
    */
   public final String toString() 
   {
	String str = "(" + x + "," + y +")";
	return str;    
   }
}
