
/**
* MACS is an Anticipatory Learning Classifier System written in Java.
* 
* Copyright (C) Pierre Grard and Olivier Sigaud with the occasional help
* of Fabien Flacher.
* Contact Olivier.Sigaud@lip6.fr for any queries, or write to
* 
* Olivier Sigaud
* AnimatLab
* Laboratoire d'Informatique de Paris 6
* 8, rue du Capitaine Scott,
* 75015 PARIS, FRANCE
* 
* This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
* 
* More informations about the GNU General Public License can be found
* at the following web site:
* http://www.opensource.org/licenses/gpl-license.php
**/

package Environments;

import java.util.ArrayList;
import java.util.Vector;

/**
 * The Environment class is an abstract class which must be overriden to implement 
 * an actual environment.
 * @author Pierre GERARD (pierre.gerard@lip6.fr)
 */
public abstract class Environment 
{
   protected String input;
   protected String output;
   protected double reward;
   protected boolean goalReached;
   protected boolean firstStep;
   protected String representativeRules;
   protected int nbRepresentativeRules;
   protected String listOfPerceptions;
   protected String coordsOfPerceptions;
   
   Environment() 
   {
	goalReached = false;
	firstStep = true;    
   }
   
   /**
    * @return String
    */
   public final String getOutput() 
   {
	//System.out.println(output);
	return output;    
   }
   
   /**
    * @return double
    */
   public final double getReward() 
   {
	return reward;    
   }
   
   /**
    * @return boolean
    */
   public final boolean isGoalReached() 
   {
	return goalReached;    
   }
   
   /**
    * @return String
    */
   public String getRulesForm() 
   {
	return getOutputForm() + " " + getInputForm();    
   }
   
   /**
    * @param newInput
    */
   public final void setInput(String newInput) 
   {
	//System.out.println(newInput);
	input = newInput;    
   }
   
   public final void takeStep() 
   {
	if (firstStep)
	    {
		output = computeFirstOutputs();
		firstStep = false;
	    }
	else   output = computeNewOutputs();    
   }
   
   /**
    * @return String
    */
   public final String getListOfPerceptions() 
   {
	return listOfPerceptions;    
   }
   
   /**
    * @return String
    */
   public final String getCoordsOfPerceptions() 
   {
	return coordsOfPerceptions;    
   }
   
   /**
    * @return String
    */
   public final String getRepresentativeRules() 
   {
	return representativeRules;    
   }
   
   /**
    * @return int
    */
   public final int getNbRepresentativeRules() 
   {
	return nbRepresentativeRules;    
   }
   
   public abstract String computeNewOutputs();
   
   public abstract String computeFirstOutputs();
    public abstract void changeCellValue(int i, int j, int value);
   /**
    * @return String
    */
   public abstract String getOutputForm();
   
   /**
    * @return String
    */
   public abstract String getInputForm();
   
   /**
    * @return String
    */
   public abstract String getStepInformation();
   
   /**
    * @return java.util.ArrayList
    */
   public abstract Vector prepareForNewProblem();
   
   /**
    * @return String
    */
   public abstract String toString();
}// end of Environment
