
/**
* MACS is an Anticipatory Learning Classifier System written in Java.
* 
* Copyright (C) Pierre Grard and Olivier Sigaud with the occasional help
* of Fabien Flacher.
* Contact Olivier.Sigaud@lip6.fr for any queries, or write to
* 
* Olivier Sigaud
* AnimatLab
* Laboratoire d'Informatique de Paris 6
* 8, rue du Capitaine Scott,
* 75015 PARIS, FRANCE
* 
* This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
* 
* More informations about the GNU General Public License can be found
* at the following web site:
* http://www.opensource.org/licenses/gpl-license.php
**/

package Environments;

import java.util.Vector;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.BufferedReader;

import Utils.StaticRandom;

import MACS.RuleBasedSystem;

import Experiments.Main;
import Experiments.ExperimentsParameters;
/**
 * The ModelManager class is used to deal with MVC modelling
 * in the graphical version of MACS
 * @author Olivier SIGAUD (olivier.sigaud@lip6.fr)
 */
public class ModelManager
{
   /**
    * Flag indicating the stop event reception of the Environment
    */
   protected boolean myStoppedFlag;
   private RuleBasedSystem mySystem;
   private Environment myEnv;
   
   /**
    * Constructor
    */
   public ModelManager() { }
   
   /**
    * @return Experiments.RuleBasedSystem
    */
   protected RuleBasedSystem getRuleBasedSystem() 
   {
	return mySystem;    
   }
   
   /**
    * Set the Flag indicating the stop event reception of the Environment
    * @param isStopped
    */
   public final void setStoppedFlag(boolean isStopped) 
   {
        myStoppedFlag = isStopped ;    
   }
   
   /**
    * Get the Flag indicating the stop event reception of the Environment
    * @return boolean
    */
   public final boolean isStopped() 
   {
        return myStoppedFlag ;    
   }
   
   /**
    * @param env
    */
   public void setEnvironment(Environment env) 
   {
	myEnv = env;    
   }
   
   /**
    * @param syst
    */
   public void setRuleBasedSystem(RuleBasedSystem syst) 
   {
	mySystem = syst;    
   }
   
   /**
    * Compute the next step of the Environment for a GUI output
    */
   public void takeStepForGUI() 
   {
	myEnv.takeStep();

	myEnv.setInput(getRuleBasedSystem().run(myEnv.getReward(),myEnv.getOutput(),myEnv.isGoalReached()));
   }
}
