
/**
* MACS is an Anticipatory Learning Classifier System written in Java.
* 
* Copyright (C) Pierre Grard and Olivier Sigaud with the occasional help
* of Fabien Flacher.
* Contact Olivier.Sigaud@lip6.fr for any queries, or write to
* 
* Olivier Sigaud
* AnimatLab
* Laboratoire d'Informatique de Paris 6
* 8, rue du Capitaine Scott,
* 75015 PARIS, FRANCE
* 
* This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
* 
* More informations about the GNU General Public License can be found
* at the following web site:
* http://www.opensource.org/licenses/gpl-license.php
**/

package Environments;

import Utils.StaticRandom;
import Utils.Misc;

/**
 * This class is used to represent a Woods Environment 
 * where the agent perceives 25 cells around itself.
* @author <a href="mailto:Olivier.Sigaud@lip6.fr"> Olivier Sigaud</a> and <a href="mailto:gerpy@free.fr"> Pierre Grard</a>.
 */

public class Woods25 extends Woods 
{
   /**
    * @param fileName
    */
   public Woods25(String fileName) 
   {
	agent = new Agent25();
	if (!fileName.equals("none"))	loadWoods25(fileName);
	else if (fixNbWalls)		generateWoods25(defaultNbWalls);
	else				generateWoods25(defaultProbaWall);

	//System.out.println(toString());    
   }
   
   /**
    * @param fileName
    */
   protected final void loadWoods25(String fileName) 
   {
       //setStart(2,4);
	setStart(5,7);

	loader = new WoodsLoader(this);
	Cell[][] tempomap = loader.fillWood(fileName);
	setMaps(tempomap);

	if (xStart == -1)	chooseStartAtRandom = true;
	else			chooseStartAtRandom = false;

	additionnalBits = new short[nbIrrelevantBits + nbRandomBits];

	fillActionList();

	updateRepresentativeRules();
	updateListOfPerceptions();

	//System.out.println(representativeRules);

	prepareForNewProblem();    
   }
   
   /**
    * nb : devra etre modifi : le generator doit changer...
    * @param probaWalls
    */
   protected final void generateWoods25(double probaWalls) 
   {
	super.generateWoods(probaWalls);    
   }
   
   /**
    * @param nbWalls
    */
   protected final void generateWoods25(short nbWalls) 
   {
	super.generateWoods(nbWalls);    
   }
   
   protected final void finalizeWoodsGeneration() 
   {
       //	setStart(2,4);
	setStart(5,7);
	chooseStartAtRandom = false;

	nbSubGoalsToReach=(short)1;
	nbIrrelevantBits=(byte)0;
	nbRandomBits=(byte)0;
	additionnalBits = new short[nbIrrelevantBits + nbRandomBits];

	fillActionList();

	updateRepresentativeRules();
	updateListOfPerceptions();

	//System.out.println(representativeRules);

	prepareForNewProblem();    
   }
   
   protected final void fillActionList() 
   {
	actionList.add("[0]");
	actionList.add("[1]");
	actionList.add("[2]");
	actionList.add("[3]");    
   }
   
   public String computeNewOutputs() 
   {
	return computeNewOutputs(false);    
   }
   
   protected final String computeOutput(short x, short y) 
   {
	String str = new String("[");

	str = str + map[x-2][y-2] + "|";
	str = str + map[x-2][y-1] + "|";
	str = str + map[x-2][y] + "|";
	str = str + map[x-2][y+1] + "|";
	str = str + map[x-2][y+2] + "|";

	str = str + map[x-1][y-2] + "|";
	str = str + map[x-1][y-1] + "|";
	str = str + map[x-1][y] + "|";
	str = str + map[x-1][y+1] + "|";
	str = str + map[x-1][y+2] + "|";

	str = str + map[x][y-2] + "|";
	str = str + map[x][y-1] + "|";
	str = str + map[x][y] + "|";
	str = str + map[x][y+1] + "|";
	str = str + map[x][y+2] + "|";

	str = str + map[x+1][y-2] + "|";
	str = str + map[x+1][y-1] + "|";
	str = str + map[x+1][y] + "|";
	str = str + map[x+1][y+1] + "|";
	str = str + map[x+1][y+2] + "|";

	str = str + map[x+2][y-2] + "|";
	str = str + map[x+2][y-1] + "|";
	str = str + map[x+2][y] + "|";
	str = str + map[x+2][y+1] + "|";
	str = str + map[x+2][y+2];

	for (short i=0; i<(nbIrrelevantBits+nbRandomBits) ;i++)
	    str = str + "|" + additionnalBits[i];
	
        return str + "]";
   }
   
   /**
    * @return String
    */
   public final String getOutputForm() 
   {
	String form = new String();

	form = form + "[3|3|3|3|3|3|3|3|3|3|3|3|3|3|3|3|3|3|3|3|3|3|3|3|3";
	
	for (short i=0 ; i<nbIrrelevantBits ; i++)
		form = form + "|2";

	for (short i=0 ; i<nbRandomBits ; i++)
		form = form + "|2";

	form = form +"]";

	return form;    
   }
   
   /**
    * @return String
    */
   public final String getInputForm() 
   {
	return "[4]";    
   }
   
   /**
    * @return String
    */
   public final String getStepInformation() 
   {
	String str = new String();

	char actionSymbol = 'S';
	if (action==0) actionSymbol = '0';
	else if (action==1) actionSymbol = '1';
	else if (action==2) actionSymbol = '2';
	else if (action==3) actionSymbol = '3';

	String strLine = new String();
	strLine = "+";
	for (short i=0 ; i<xSize ; i++) strLine = strLine + " ";
	strLine = strLine + "+";

	str = strLine + "\n" + str;
	for (short j=0 ; j<ySize ; j++)
	    {
		str = str + " ";
		for (short i=0 ; i<xSize ; i++)
		    {
			if (agent.isIn(i,j))
			    str = str + actionSymbol;
			else if (map[i][j].isFreeCell())
			    str = str + " ";
			else if (map[i][j].isWallCell())
			    str = str + "#";
			else if (map[i][j].isFoodCell())
			    str = str + "X";	
		    }
		str = str + " ";
		if (j!=ySize) str = str + "\n";
	    }
	str = str + "\n" + strLine;

	str = Misc.putSideBySide("State : ", str, (short)12);

	str = "            Perception " + output + "\n" + str;
	str = "Outputs :   Reward     " + reward + "\n" + str;
	str = "Inputs :    Action     " + input + "\n" + str;

	return str;    
   }
}
