package Environments;

import Utils.StaticRandom;
import Utils.Misc;

/**
 * This class is used to represent a Woods Environment 
 * where the agent perceives the distance to walls in the four directions.
* @author <a href="mailto:Olivier.Sigaud@lip6.fr"> Olivier Sigaud</a> and <a href="mailto:gerpy@free.fr"> Pierre Grard</a>.
 */

public class WoodsDist extends Woods 
{
   /**
    * @param fileName
    */
   public WoodsDist(String fileName) 
   {
	agent = new Agent25();
	if (!fileName.equals("none"))	loadWoodsDist(fileName);
	else if (fixNbWalls)		generateWoodsDist(defaultNbWalls);
	else				generateWoodsDist(defaultProbaWall);

	//System.out.println(toString());    
   }
   
   /**
    * @param fileName
    */
   protected final void loadWoodsDist(String fileName) 
   {
       //setStart(2,4);
	setStart(5,7);

	loader = new WoodsLoader(this);
	Cell[][] tempomap = loader.fillWood(fileName);
	setMaps(tempomap);

	if (xStart == -1)	chooseStartAtRandom = true;
	else			chooseStartAtRandom = false;

	additionnalBits = new short[nbIrrelevantBits + nbRandomBits];

	fillActionList();

	updateRepresentativeRules();
	updateListOfPerceptions();

	//System.out.println(representativeRules);

	prepareForNewProblem();    
   }
   
   /**
    * nb : devra etre modifi : le generator doit changer...
    * @param probaWalls
    */
   protected final void generateWoodsDist(double probaWalls) 
   {
	super.generateWoods(probaWalls);    
   }
   
   /**
    * @param nbWalls
    */
   protected final void generateWoodsDist(short nbWalls) 
   {
	super.generateWoods(nbWalls);    
   }
   
   protected final void finalizeWoodsGeneration() 
   {
       //	setStart(2,4);
	setStart(5,7);
	chooseStartAtRandom = false;

	nbSubGoalsToReach=(short)1;
	nbIrrelevantBits=(byte)0;
	nbRandomBits=(byte)0;
	additionnalBits = new short[nbIrrelevantBits + nbRandomBits];

	fillActionList();

	updateRepresentativeRules();
	updateListOfPerceptions();

	//System.out.println(representativeRules);

	prepareForNewProblem();    
   }
   
   protected final void fillActionList() 
   {
	actionList.add("[0]");
	actionList.add("[1]");
	actionList.add("[2]");
	actionList.add("[3]");    
   }
   
   public String computeNewOutputs() 
   {
	return computeNewOutputs(false);    
   }
   
   protected final String computeOutput(short x, short y) 
   {
	String str = new String("[");

	str = str + getDistNorth() + "|";
	str = str + getDistEast() + "|";
	str = str + getDistSouth() + "|";
	str = str + getDistWest();

       for (short i=0; i<(nbIrrelevantBits+nbRandomBits) ;i++)
	    str = str + "|" + additionnalBits[i];
	
        return str + "]";
   }
   
   /**
    * @return String
    */
   public final String getOutputForm() 
   {
	String form = new String();

	form = form + "[7|10|7|10";
	
	for (short i=0 ; i<nbIrrelevantBits ; i++)
		form = form + "|2";

	for (short i=0 ; i<nbRandomBits ; i++)
		form = form + "|2";

	form = form +"]";

	return form;    
   }
   
   /**
    * @return String
    */
   public final String getInputForm() 
   {
	return "[4]";    
   }
public final String getDistNorth()
{
    int i=0;
    while (!map[agent.getXI()][agent.getYI()-i].isWallCell()) i++;
    return ""+i;
}
public final String getDistSouth()
{
    int i=0;
    while (!map[agent.getXI()][agent.getYI()+i].isWallCell()) i++;
    return ""+i;
}
public final String getDistEast()
{
    int i=0;
    while (!map[agent.getXI()+i][agent.getYI()].isWallCell()) i++;
    return ""+i;
}
public final String getDistWest()
{
    int i=0;
    while (!map[agent.getXI()-i][agent.getYI()].isWallCell()) i++;
    return ""+i;
}
   /**
    * @return String
    */
   public final String getStepInformation() 
   {
	String str = new String();

	char actionSymbol = 'S';
	if (action==0) actionSymbol = '0';
	else if (action==1) actionSymbol = '1';
	else if (action==2) actionSymbol = '2';
	else if (action==3) actionSymbol = '3';

	String strLine = new String();
	strLine = "+";
	for (short i=0 ; i<xSize ; i++) strLine = strLine + " ";
	strLine = strLine + "+";

	str = strLine + "\n" + str;
	for (short j=0 ; j<ySize ; j++)
	    {
		str = str + " ";
		for (short i=0 ; i<xSize ; i++)
		    {
			if (agent.isIn(i,j))
			    str = str + actionSymbol;
			else if (map[i][j].isFreeCell())
			    str = str + " ";
			else if (map[i][j].isWallCell())
			    str = str + "#";
			else if (map[i][j].isFoodCell())
			    str = str + "X";	
		    }
		str = str + " ";
		if (j!=ySize) str = str + "\n";
	    }
	str = str + "\n" + strLine;

	str = Misc.putSideBySide("State : ", str, (short)12);

	str = "            Perception " + output + "\n" + str;
	str = "Outputs :   Reward     " + reward + "\n" + str;
	str = "Inputs :    Action     " + input + "\n" + str;

	return str;    
   }
}
