
/**
* MACS is an Anticipatory Learning Classifier System written in Java.
* 
* Copyright (C) Pierre Grard and Olivier Sigaud with the occasional help
* of Fabien Flacher.
* Contact Olivier.Sigaud@lip6.fr for any queries, or write to
* 
* Olivier Sigaud
* AnimatLab
* Laboratoire d'Informatique de Paris 6
* 8, rue du Capitaine Scott,
* 75015 PARIS, FRANCE
* 
* This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
* 
* More informations about the GNU General Public License can be found
* at the following web site:
* http://www.opensource.org/licenses/gpl-license.php
**/

package Environments;

import Utils.Misc;
import java.io.FileReader;
import java.io.BufferedReader;

/**
 * This class is used to load from a file a standard Woods Environment 
* @author <a href="mailto:Olivier.Sigaud@lip6.fr"> Olivier Sigaud</a> and <a href="mailto:gerpy@free.fr"> Pierre Grard</a>.
 */

class WoodsLoader 
{
   private int x = 0;
   private int y = 0;
   private int xMax = 0;
   private Woods Woods;
    private    Cell map[][];

   
   /**
    * @param fileName
    * @param ww
    */
   WoodsLoader(Woods ww) 
   {
	Woods = ww;
   }
   
   protected final void setFree() 
   {
	Woods = null;    
   }
   
   /**
    * @param fn
    */
   protected final Cell[][] fillWood(String fn) 
   {
       try
	   {
	       Cell retour[][];
	       map = new Cell[256][256];

	       FileReader fileReader = new FileReader(fn);
	       BufferedReader buffReader = new BufferedReader(fileReader);
	       String line = new String();
	       int nblines = 0;
	       while (buffReader.ready())
		   {
		       line = buffReader.readLine();
		       if (line.startsWith("\""))
			   {
			       addLine(line.substring(1, line.length()-1));
			       nblines++;
			   }
		       else
			   checkForParameter(line);
		   }
	       /*
	       for (int i=0 ; i<xMax ; i++)
		   {
		       for (int j=0 ; j<nblines ; j++)
			   {
			       map[i][j].setY(nblines-j);
			   }
		   }
	       */
	       fileReader.close();
	       retour = new Cell[xMax][nblines];
	       for (int i=0 ; i<xMax ; i++)
		   {
		       for (int j=0 ; j<nblines ; j++)
			   {
			       retour[i][j] = map[i][j];
			   }
		   }
	       return retour;
	   }
	catch (Exception e) 
	    {
		e.printStackTrace();
		System.out.println (e.toString());
	       return null;
	    }    
   }
   
   /**
    * @param line
    */
   private final void addLine(String line) 
   {
       if (line.length()>xMax) xMax = line.length();
	for (x=0 ; x<line.length() ; x++)
	    {
		if (line.charAt(x)=='S')
		    {
			Woods.setStart(x,y);
			map[x][y] = new Cell(0,x,y);
		    }
		else if (line.charAt(x)==' ')
		    map[x][y] = new Cell(0,x,y);
		else if (line.charAt(x)=='#')
		    map[x][y] = new Cell(1,x,y);
		else if (line.charAt(x)=='X')
		    map[x][y] = new Cell(2,x,y);
	    }
	y++;
   }
   
   /**
    * @param line
    */
   private final void checkForParameter(String line) 
   {
	line = Misc.removeSpaces(line);
	if (line.lastIndexOf("//")!=-1) line = line.substring(0, line.lastIndexOf("//"));
	if ((!line.equals("")) && (!line.startsWith("//")) && (line.indexOf('=')!=-1))
	    {
		String param = line.substring(0, line.indexOf('='));
		String value = line.substring(line.indexOf('=')+1);
		if (param.equals("nbSubGoalsToReach"))
		    Woods.setNbSubGoalsToReach(Short.valueOf(value).shortValue());
		if (param.equals("nbIrrelevantBits"))
		    Woods.setNbIrrelevantBits(Short.valueOf(value).shortValue());
		if (param.equals("nbRandomBits"))
		    Woods.setNbRandomBits(Short.valueOf(value).shortValue());
	    }    
   }
}
