package Experiments;

import java.util.Random;
import Utils.ParameterSet;

public class ExperimentsParameters 
{
   public static String localPath = "/local/home/sigaud/osd/WebMACS/MACS/";
   public String system;
   public String systemConfig;
   public String systemInit;
   public String environmentConfig;
   public String outputDirectory;
   public String environment;
   public String experienceDesignation;
   public long seed;
   public int nbSimulations;
   public int startSimuNumber;
   public int nbTrials;
   public int nbStepsBeforeInterrupt;
   public int nbStepsBeforeChange;
   public int nbTrialsOfRandomWalk;
   public int nbStepsOfRandomWalk;
   public int knowledgeComputationFrequency;
   public boolean fixedNbTimeSteps;
   public boolean differentDirectories;
   public boolean verboseTraceInFile;
   public boolean verboseTraceOnTerm;
   public boolean tinyTraceOnTerm;
   public boolean reportsForGnuplot;
   public boolean storeFinalSet;
   public boolean storeEnvironment;
   public boolean traceTime;
   public boolean traceKnowledge;
   public boolean traceCumulatedReward;
   
   /**
    * @param fileName
    */
   public ExperimentsParameters(String fileName) 
   {	
	ParameterSet parameters = new ParameterSet(fileName);

	system			= parameters.getValue("system");

	systemConfig            = localPath + parameters.getValue("systemConfig");
	systemInit              = localPath + parameters.getValue("systemInit");
	environmentConfig       = localPath + parameters.getValue("environmentConfig");
	outputDirectory         = localPath + parameters.getValue("outputDirectory");

	environment             = parameters.getValue("environment");
	seed                    = computeSeed(parameters.getValue("seed"));

	nbSimulations           = Integer.valueOf(parameters.getValue("nbSimulations")).intValue(); 
        startSimuNumber         = Integer.valueOf(parameters.getValue("startSimuNumber")).intValue(); 

	nbTrials                = Integer.valueOf(parameters.getValue("nbTrials")).intValue(); 
	fixedNbTimeSteps        = convertSwitch(parameters.getValue("fixedNbTimeSteps"));
	nbStepsBeforeInterrupt  = Integer.valueOf(parameters.getValue("nbStepsBeforeInterrupt")).intValue();  
	nbStepsBeforeChange     = Integer.valueOf(parameters.getValue("nbStepsBeforeChange")).intValue();  

	nbTrialsOfRandomWalk    = Integer.valueOf(parameters.getValue("nbTrialsOfRandomWalk")).intValue();
	nbStepsOfRandomWalk     = Integer.valueOf(parameters.getValue("nbStepsOfRandomWalk")).intValue();

	experienceDesignation   = parameters.getValue("experienceDesignation"); 
	differentDirectories    = convertSwitch(parameters.getValue("differentDirectories"));

	verboseTraceInFile      = convertSwitch(parameters.getValue("verboseTraceInFile"));
	verboseTraceOnTerm      = convertSwitch(parameters.getValue("verboseTraceOnTerm"));
	tinyTraceOnTerm         = convertSwitch(parameters.getValue("tinyTraceOnTerm")) && (!verboseTraceOnTerm);
	reportsForGnuplot       = convertSwitch(parameters.getValue("reportsForGnuplot"));
	storeFinalSet           = convertSwitch(parameters.getValue("storeFinalSet"));
	storeEnvironment        = convertSwitch(parameters.getValue("storeEnvironment"));

	traceTime               = convertSwitch(parameters.getValue("traceTime"));

	traceKnowledge          = convertSwitch(parameters.getValue("traceKnowledge"));
	traceCumulatedReward    = convertSwitch(parameters.getValue("traceCumulatedReward"));
	knowledgeComputationFrequency        = Integer.valueOf(parameters.getValue("knowledgeComputationFrequency")).intValue();    
   }
   
   /**
    * @param str
    * @return boolean
    */
   private static boolean convertSwitch(String str) 
   {
	return (str.equals("on"));    
   }
   
   /**
    * @param strSeed
    * @return long
    */
   private long computeSeed(String strSeed) 
   {
 	if (strSeed.equals("none"))	return (long)(new Random().nextDouble() * (Byte.MAX_VALUE*2+1));
 	else				return Long.valueOf(strSeed).longValue();    
   }
}
