
/**
 * MACS is an Anticipatory Learning Classifier System written in Java.
 * 
 * Copyright (C) Pierre Grard and Olivier Sigaud with the occasional help
 * of Fabien Flacher.
 * Contact Olivier.Sigaud@lip6.fr for any queries, or write to
 * 
 * Olivier Sigaud
 * AnimatLab
 * Laboratoire d'Informatique de Paris 6
 * 8, rue du Capitaine Scott,
 * 75015 PARIS, FRANCE
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * More informations about the GNU General Public License can be found
 * at the following web site:
 * http://www.opensource.org/licenses/gpl-license.php
 **/

package Experiments;

import java.io.FileReader;
import java.io.FileWriter;
import java.io.BufferedReader;
import java.lang.reflect.Constructor;

import Environments.Environment;
import MACS.RuleBasedSystem;
import MACS.ClassifierSystem;
import Environments.ModelManager;
import Utils.StaticRandom;
import Utils.Chrono;

/**
 * This is the main class to run experiments.
 * @author <a href="mailto:Olivier.Sigaud@lip6.fr"> Olivier Sigaud</a> and <a href="mailto:gerpy@free.fr"> Pierre Grard</a>.
 * 
 **/

public class Main 
{
    public static ExperimentsParameters parameters;
   
    /**
     * @return String
     */
    public static String loadInitString() 
    {
	String initString = "";

	try
	    {
		BufferedReader buffReader = new BufferedReader(new FileReader(parameters.systemInit));
		initString = "";
		boolean begin = true;
		while(buffReader.ready())
		    {
			if(begin)
			    initString = buffReader.readLine();
			else
			    initString = initString + "\n" + buffReader.readLine();
			begin = false;
		    }
		buffReader.close();
	    }  catch(Exception e) {e.printStackTrace();System.exit(-1);}
	
	return initString;    
    }
   
    /**
     * @return Environments.Environment
     */
    public static Environment createEnvironment() 
    {
	Environment env = null;
	try
	    {
		Class[] params = new Class[1];
		params[0] = Class.forName("java.lang.String");
		Constructor construct = Class.forName("Environments." + parameters.environment).getConstructor(params);
		String[] params2 = new String[1];
		params2[0] = parameters.environmentConfig;
		env =(Environment)(construct.newInstance(params2));
	    }
	catch(Exception e) {e.printStackTrace(); System.exit(-1);}
	return env;    
    }
   
    /**
     * @param argv[]
     */
    public static void main(String args[]) 
    {
	System.out.println("("+args[0]+")");
	parameters = new ExperimentsParameters(args[0]);
	StaticRandom.setSeed(parameters.seed);

	Environment env = createEnvironment();
	RuleBasedSystem system = null;

	if (parameters.system.equals("MACS"))
	    {
		system = new MACS.ModularClassifierSystem(env.getRulesForm(), loadInitString(), parameters.systemConfig);
		((MACS.ModularClassifierSystem)system).getTraceManager().setOrderedPerceptions(env.getListOfPerceptions());
		((MACS.ModularClassifierSystem)system).getTraceManager().setCoordsOfOrderedPerceptions(env.getCoordsOfPerceptions());
	    }

	env.prepareForNewProblem();
	system.prepareForNewProblem();
	Chrono c = new Chrono();
	Sequencer seq = new Sequencer(env,system);

	if (!parameters.fixedNbTimeSteps)
	    seq.run(parameters.nbSimulations, parameters.nbTrials, parameters.nbStepsBeforeInterrupt);
	else
	    seq.run(parameters.nbSimulations, parameters.nbStepsBeforeInterrupt);
	c.stop();
    }
}
