
/**
* MACS is an Anticipatory Learning Classifier System written in Java.
* 
* Copyright (C) Pierre Grard and Olivier Sigaud with the occasional help
* of Fabien Flacher.
* Contact Olivier.Sigaud@lip6.fr for any queries, or write to
* 
* Olivier Sigaud
* AnimatLab
* Laboratoire d'Informatique de Paris 6
* 8, rue du Capitaine Scott,
* 75015 PARIS, FRANCE
* 
* This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
* 
* More informations about the GNU General Public License can be found
* at the following web site:
* http://www.opensource.org/licenses/gpl-license.php
**/

package MACS;

import java.util.*;

import Classifier.Perception;
import Classifier.Action;
import Classifier.Transition;

/**
 * The ActionTable class is in charge of storing the number of instances of actions
 * performed for each action for each known perception
* @author <a href="mailto:Olivier.Sigaud@lip6.fr"> Olivier Sigaud</a> and <a href="mailto:gerpy@free.fr"> Pierre Grard</a>.
* 
 */
class ActionTable
{
    private int memory[];

    public ActionTable(int nbActions)
    {
	memory = new int[nbActions];
    }
    public int getNbActions(int index)
    {
	return memory[index];
    }
    public void addAction(byte index)
    {
	memory[index]++;
    }
    public int getBooleanReward()
    {
	for (int i=0;i<memory.length;i++)
	    {
		if (memory[i]==0) return 1;
	    }
	return 0;
    }
    public int getPreciseReward()
    {
	int sum = 0;
	for (int i=0;i<memory.length;i++)
	    {
		sum += Math.max(5-memory[i],0);
	    }
	return sum;
    }
    public String toString()
    {
	String retour = "[";
	for (int i=0;i<memory.length;i++)
	    {
		retour = retour + memory[i] +" ";
	    }
	return retour + "]";
    }

}
