
/**
* MACS is an Anticipatory Learning Classifier System written in Java.
* 
* Copyright (C) Pierre Grard and Olivier Sigaud with the occasional help
* of Fabien Flacher.
* Contact Olivier.Sigaud@lip6.fr for any queries, or write to
* 
* Olivier Sigaud
* AnimatLab
* Laboratoire d'Informatique de Paris 6
* 8, rue du Capitaine Scott,
* 75015 PARIS, FRANCE
* 
* This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
* 
* More informations about the GNU General Public License can be found
* at the following web site:
* http://www.opensource.org/licenses/gpl-license.php
**/

package MACS;

import Classifier.AnticipatoryClassifier;
import Classifier.Perception;
import Classifier.Action;

/**
 * This class is used to represent the AnticipationLearningManager in MACS.
* @author <a href="mailto:Olivier.Sigaud@lip6.fr"> Olivier Sigaud</a> and <a href="mailto:gerpy@free.fr"> Pierre Grard</a>.
* 
 */
final class AnticipationLearningManager
{
    private ClassifierSystem classifierSystem;

    protected AnticipationLearningManager(ClassifierSystem cs)
    {
	classifierSystem = cs;
    }

    private void checkAnticipation (AnticipatoryClassifier classifier, Perception lastPerception, Perception currentPerception)
    {
	if ( classifier.matchAnticipation(currentPerception) && classifier.matchCondition(lastPerception))
	    classifier.addGoodAnticipation(lastPerception,ClassifierSystemParameters.learningRate);
	else if (!classifier.matchAnticipation(currentPerception) && classifier.matchCondition(lastPerception))
	    anticipatesBadly(classifier, lastPerception);
	else
	    classifier.increaseLastEvaluation();
    }
    private void anticipatesBadly(AnticipatoryClassifier classifier, Perception lastPerception)
    {
	classifier.addBadAnticipation(lastPerception,ClassifierSystemParameters.learningRate);
	/*
	if (classifier.doesAnticipateBadly(ClassifierSystemParameters.nbFailuresToRemove))
	    {
		System.out.println("------------- classifier removed !!! ---------------");
		classifierSystem.removeClassifier(classifier);
		if (ClassifierSystemParameters.traceAnticipationSelection) classifierSystem.getTraceManager().getClassifiersRemovedByAnticipationSelection().addClassifier(classifier, false);
	    }
	*/
    }

    protected final void run (Perception lastPerception, Action lastAction, Perception currentPerception)
    {
	//System.out.println("------------- check Anticipation ---------------");

	for (short i=0; i<classifierSystem.getNbClassifiers() ; i++)
	    {
		AnticipatoryClassifier classifier = (AnticipatoryClassifier)classifierSystem.getClassifier(i);
		if (classifier.matchAction(lastAction))
		    checkAnticipation(classifier, lastPerception, currentPerception);
	    }
	//System.out.println("---------------------------------------------");
    }
}
