
/**
* MACS is an Anticipatory Learning Classifier System written in Java.
* 
* Copyright (C) Pierre Grard and Olivier Sigaud with the occasional help
* of Fabien Flacher.
* Contact Olivier.Sigaud@lip6.fr for any queries, or write to
* 
* Olivier Sigaud
* AnimatLab
* Laboratoire d'Informatique de Paris 6
* 8, rue du Capitaine Scott,
* 75015 PARIS, FRANCE
* 
* This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
* 
* More informations about the GNU General Public License can be found
* at the following web site:
* http://www.opensource.org/licenses/gpl-license.php
**/

package MACS;

import java.util.Vector;
import Utils.StaticRandom;
import Utils.ValuesDistribution;
import Classifier.Action;
import Classifier.Perception;
import Classifier.BasicClassifier;
/**
 * This class is used to represent a list of classifiers.
* @author <a href="mailto:Olivier.Sigaud@lip6.fr"> Olivier Sigaud</a> and <a href="mailto:gerpy@free.fr"> Pierre Grard</a>.
* 
 */

public class ClassifierList extends Vector 
{
   
   /**
    * @param cset
    */
   public ClassifierList(ClassifierList cset) 
   {
	for (short i=0; i<cset.getNbClassifiers(); ++i)
	    addElement((BasicClassifier)(cset.getClassifier(i)).clone());    
   }
   
   public ClassifierList() 
   {
    
   }
   
   /**
    * @return Utils.ValuesDistribution
   public ValuesDistribution buildSpecDistribution() 
   {
	ValuesDistribution specDistribution = new ValuesDistribution(getNbClassifiers());

	for (short i=0; i<getNbClassifiers(); ++i)
	    specDistribution.add(getClassifier(i).getCondition().getSpecialization());

	return specDistribution;    
   }
    */
   
   /**
    * @param index
    * @return Classifier.BasicClassifier
    */
   public BasicClassifier getClassifier(short index) 
   {
	return (BasicClassifier)get(index);    
   }
   
   /**
    * @param index
    * @param classifier
    */
   public void setClassifier(short index, BasicClassifier classifier) 
   {
	setElementAt(classifier, index);    
   }
   
   /**
    * @param classifier
    * @return short
    */
   public short getIndexOfClassifier(BasicClassifier classifier) 
   {
	return (short)(indexOf(classifier));    
   }
   
   /**
    * @return short
    */
   public short getNbClassifiers() 
   {
	return (short)size();    
   }
   
   /**
    * @param index
    * @return String
    */
   public String getActionString(short index) 
   {
	return (getClassifier(index).getActionString());    
   }
   
   /**
    * @param index
    * @return String
    */
   public String getPerceptionString(short index) 
   {
	return (getClassifier(index).getPerceptionString());    
   }
   
   /**
    * Note : this method is dangerous 
    * contains depends on whether the classifier passed is Basic or Anticipatory
    * 
    * @param classifier
    * @param verifyExistence
    * @return boolean
    */
   public boolean addClassifier(BasicClassifier classifier, boolean verifyExistence) 
   {
	boolean contains = false;
	if ((!verifyExistence) || (!(contains=contains(classifier))))	addElement(classifier);
	return contains;    
   }
   
   /**
    * @param classifier
    */
   public void removeClassifier(BasicClassifier classifier) 
   {
	removeElement(classifier);    
   }
   
   /**
    * @param index
    */
   public void removeClassifier(short index) 
   {
	removeElementAt(index);    
   }
   
   /**
    * @param classifierList
    */
   public void removeClassifiersOf(ClassifierList classifierList) 
   {
	for (short i=0 ; i<classifierList.getNbClassifiers() ; i++)
	    removeElement(classifierList.getClassifier(i));    
   }
   
   /**
    * @param newClassifier
    * @return boolean
    */
   public boolean contains(BasicClassifier newClassifier) 
   {
	for (short i=0; i<size(); i++)
	    if (getClassifier(i).equals(newClassifier))
		return true;

	return false;    
   }
   
   /**
    * @param newClassifier
    * @return boolean
    */
   public boolean containsClassifierMatching(BasicClassifier newClassifier) 
   {
	for (short i=0; i<getNbClassifiers(); i++)
	    if (getClassifier(i).matches(newClassifier)) return true;
	return false;    
   }
   
   /**
    * @param situation
    * @return boolean
    */
   public boolean containsClassifierMatching(Perception situation) 
   {
	for (short i=0; i<getNbClassifiers(); i++)
	    if (getClassifier(i).matchCondition(situation))
		return true;

	return false;    
   }
   
   /**
    * @param situation
    * @return boolean
    */
   public boolean containsClassifierNotMatching(Perception situation) 
   {
	for (short i=0; i<getNbClassifiers(); i++)
	    if (!getClassifier(i).matchCondition(situation))
		return true;

	return false;    
   }
   
   /**
    * @param newClassifier
    * @return boolean
    */
   public boolean containsLessGeneralThan(BasicClassifier newClassifier) 
   {
	for (short i=0; i<getNbClassifiers(); i++)
	    if (getClassifier(i).isLessGeneralThan(newClassifier))
		return true;
	
	return false;    
   }
   
   /**
    * @param newClassifier
    * @return boolean
    */
   public boolean containsMoreGeneralThan(BasicClassifier newClassifier) 
   {
	for (short i=0; i<getNbClassifiers(); i++)
	    if (getClassifier(i).isMoreGeneralThan(newClassifier))
		return true;
	
	return false;    
   }
   
   /**
    * @param newClassifier
    * @return short
    */
   public short getNbMoreSpecializedThan(BasicClassifier newClassifier) 
   {
	short nb = 0;

	for (short i=0; i<getNbClassifiers(); i++)
	    if (getClassifier(i).isLessGeneralThan(newClassifier))
		nb++;
	
	return nb;    
   }
   
   /**
    * @param newClassifier
    * @return boolean
    */
   public boolean containsEqualOrLessGeneralThan(BasicClassifier newClassifier) 
   {
	for (short i=0; i<getNbClassifiers(); i++)
	    {
		BasicClassifier classifier = getClassifier(i);
		if ( classifier.isLessGeneralThan(newClassifier)
		     || classifier.equals(newClassifier) )
		    return true;
	    }
	
	return false;    
   }
   
   /**
    * @param newClassifier
    * @return boolean
    */
   public boolean containsEqualOrMoreGeneralThan(BasicClassifier newClassifier) 
   {
	for (short i=0; i<getNbClassifiers(); i++)
	    {
		BasicClassifier classifier = getClassifier(i);
		if ( classifier.isMoreGeneralThan(newClassifier)
		     || classifier.equals(newClassifier) )
		    return true;
	    }
	
	return false;    
   }
   
   /**
    * @param newClassifier
    * @return boolean
    */
   public boolean containsComparableWith(BasicClassifier newClassifier) 
   {
	for (short i=0; i<getNbClassifiers(); i++)
	    {
		BasicClassifier classifier = getClassifier(i);
		if ( classifier.isMoreGeneralThan(newClassifier)
		     || classifier.isLessGeneralThan(newClassifier)
		     || classifier.equals(newClassifier) )
		    return true;
	    }
	
	return false;    
   }
   
   /**
    * @param classifierList
    */
   public void addClassifiersFrom(ClassifierList classifierList) 
   {
	for (short i=0 ; i<classifierList.getNbClassifiers() ; i++)
	    if (!contains(classifierList.getClassifier(i))) 
		addClassifier(classifierList.getClassifier(i), false);    
   }
   
   /**
    * @param a
    * @return ClassifierList
    */
   public ClassifierList selectSubSetForAction(Action a) 
   {
	ClassifierList newClassifierList = new ClassifierList ();

	for (short i=0 ; i<getNbClassifiers() ; i++)
	    if (getClassifier(i).getAction().equals(a))
		newClassifierList.addClassifier(getClassifier(i), false);

	return newClassifierList;    
   }
   
   /**
    * @return ClassifierList
    */
   public ClassifierList selectMostSpecializedClassifiers() 
   {
	ClassifierList specClassifiers = new ClassifierList();
	double maxSpec = 0;

	for (short i=0 ; i<getNbClassifiers() ; i++)
	    {
		if (getClassifier(i).getCondition().getSpecialization() > maxSpec)
		    {
			maxSpec = (getClassifier(i).getCondition().getSpecialization());
			specClassifiers = new ClassifierList();
		    }
	        specClassifiers.addClassifier(getClassifier(i), false);
	    }
	
	return specClassifiers;    
   }
   
   /**
    * @param perception
    * @param action
    * @return short
    */
   public short getNbClassifiersMatching(Perception perception, Action action) 
   {
	short nb = 0;

	for (short i=0 ; i<getNbClassifiers() ; i++)
	    if ( getClassifier(i).matchCondition(perception) 
		 && getClassifier(i).matchAction(action) )
		nb ++;
	
	return nb;    
   }
   
   /**
    * @param perception
    * @return short
    */
   protected final short getNbClassifiersMatching(Perception perception) 
   {
	short nb = 0;

	for (short i=0 ; i<getNbClassifiers() ; i++)
	    if ( getClassifier(i).matchCondition(perception) )
		nb ++;
	
	return nb;    
   }
   
   /**
    * @param verboseForClassifiers
    * @return String
    */
   public String toString(boolean verboseForClassifiers) 
   {
	String string = new String();

	for (short i=0 ; i<getNbClassifiers() ; i++)
	    {
		string = string + getClassifier(i).toString(verboseForClassifiers);
		if (i!=(getNbClassifiers()-1))
		    string = string + "\n";
	    }
	return string;    
   }
   
   /**
    * @return String
    */
   public String toString() 
   {
	return (toString(false));    
   }
}
