
/**
* MACS is an Anticipatory Learning Classifier System written in Java.
* 
* Copyright (C) Pierre Grard and Olivier Sigaud with the occasional help
* of Fabien Flacher.
* Contact Olivier.Sigaud@lip6.fr for any queries, or write to
* 
* Olivier Sigaud
* AnimatLab
* Laboratoire d'Informatique de Paris 6
* 8, rue du Capitaine Scott,
* 75015 PARIS, FRANCE
* 
* This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
* 
* More informations about the GNU General Public License can be found
* at the following web site:
* http://www.opensource.org/licenses/gpl-license.php
**/

package MACS;

import java.util.Vector;
import java.util.StringTokenizer;
import Utils.Misc;
import Classifier.AnticipatoryClassifier;
import Classifier.TokenRepository;
/**
 * This class is used to load a classifier system from a file.
 * Initial classifiers can be stored in a "*.ini" file,
 * but this functionality has not been tested again for a long while.
* @author <a href="mailto:Olivier.Sigaud@lip6.fr"> Olivier Sigaud</a> and <a href="mailto:gerpy@free.fr"> Pierre Grard</a>.
* 
 */
final class ClassifierSystemLoader 
{
   private ClassifierSystem classifierSystem;
   
   /**
    * @param cs
    */
   protected ClassifierSystemLoader(ClassifierSystem cs) 
   {
	classifierSystem = cs;    
   }
   
   protected final void setFree() 
   {
	classifierSystem = null;    
   }
   
   /**
    * @param rulesForm
    * @param initString
    */
   protected final void run(String rulesForm, String initString) 
   {
	setForms(rulesForm);
	formatString(initString);    
   }
   
   /**
    * @param classifierForm
    */
   protected final void setForms(String classifierForm) 
   {
	StringTokenizer stringTokenizer = new StringTokenizer(classifierForm, " ");

	classifierSystem.perceptionForm = stringTokenizer.nextToken();
	classifierSystem.actionForm = stringTokenizer.nextToken();
	TokenRepository.init(getMaxIndexInForm(classifierForm));    
   }
   
   /**
    * @param classifierForm
    * @return byte
    */
   protected final byte getMaxIndexInForm(String classifierForm) 
   {
	StringTokenizer stringTokenizer = new StringTokenizer(classifierForm, " ");

	String perceptionForm = stringTokenizer.nextToken();
	String actionForm = stringTokenizer.nextToken();
	byte index0 = getMaxIndex(perceptionForm);
	byte index1 = getMaxIndex(actionForm);
	return (byte)Math.max(index0,index1);    
   }
   
   /**
    * @param localForm
    * @return byte
    */
   public final byte getMaxIndex(String localForm) 
   {
	if (localForm.length()==3)
	    {
		String strTemp = localForm.substring(1, 2);
		byte index = Byte.valueOf(strTemp).byteValue();
		return index;
	    }
	StringTokenizer stringTokenizer = new StringTokenizer(localForm, "|");
	String strTemp;
	byte maxIndex = (byte)0;
	while (stringTokenizer.hasMoreTokens())
	    {
		strTemp = stringTokenizer.nextToken();
		if (strTemp.charAt(0)=='[') strTemp = strTemp.substring(1);
		if (strTemp.charAt(strTemp.length()-1)==']') strTemp = strTemp.substring(0, strTemp.length()-1);
		byte index = Byte.valueOf(strTemp).byteValue();
		if (index > maxIndex) maxIndex = index;
	    }
	return maxIndex;    
   }
   
   /**
    * @param initString
    * @return String
    */
   protected final String formatString(String initString) 
   {
	StringTokenizer stringTokenizer = new StringTokenizer(initString, " ");

	String strTemp = stringTokenizer.nextToken();
	strTemp = strTemp + " " + stringTokenizer.nextToken();
	strTemp = strTemp + " " + stringTokenizer.nextToken();

	return strTemp;    
   }
   
   /**
    * @param initString
    */
   protected final void fillFromString(String initString) 
   {
	if (initString!="")
	    {
		StringTokenizer stringTokenizer = new StringTokenizer(initString, "\n");
		do
		    {
			String classifierValue = stringTokenizer.nextToken();
			
			String newClassifierValue = formatString(classifierValue);
			
			AnticipatoryClassifier c1 = new AnticipatoryClassifier(ClassifierSystemParameters.systemName,newClassifierValue,classifierSystem.perceptionForm,
								       classifierSystem.actionForm,
								       ClassifierSystemParameters.initialReliability);
				classifierSystem.addClassifier(c1, true);
		    }
		while (stringTokenizer.hasMoreElements());
	    }    
   }
   
   /**
    * @param classeur
    */
   public final void addClassifier(String classeur) 
   {
	String newValue = formatString(classeur);

	AnticipatoryClassifier c1 = new AnticipatoryClassifier(ClassifierSystemParameters.systemName,newValue, classifierSystem.perceptionForm, classifierSystem.actionForm,ClassifierSystemParameters.initialReliability);
	classifierSystem.addClassifier(c1, true);    
   }
}
