
/**
* MACS is an Anticipatory Learning Classifier System written in Java.
* 
* Copyright (C) Pierre Grard and Olivier Sigaud with the occasional help
* of Fabien Flacher.
* Contact Olivier.Sigaud@lip6.fr for any queries, or write to
* 
* Olivier Sigaud
* AnimatLab
* Laboratoire d'Informatique de Paris 6
* 8, rue du Capitaine Scott,
* 75015 PARIS, FRANCE
* 
* This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
* 
* More informations about the GNU General Public License can be found
* at the following web site:
* http://www.opensource.org/licenses/gpl-license.php
**/

package MACS;

import java.util.*;
import Classifier.Perception;
/**
 * This class implements a pair <Perception,value> to store models of payoffs in TreeMaps.
* @author <a href="mailto:Olivier.Sigaud@lip6.fr"> Olivier Sigaud</a> and <a href="mailto:gerpy@free.fr"> Pierre Grard</a>.
* 
*/
final class CouplePF implements Comparable 
{
   private double second;
   private Perception first;
   
   /**
    * @param p
    * @param s
    */
   public CouplePF(Perception p, double s) 
   {
	setFirst(p);
	setSecond(s);    
   }
   
   /**
    * @param p
    */
   public CouplePF(Perception p) 
   {
	setFirst(p);    
   }
   
   /**
    * @return Classifier.Perception
    */
   public Perception getFirst() 
   {
	return first;    
   }
   
   /**
    * @param f
    */
   public void setFirst(Perception f) 
   {
	first = f;    
   }
   
   /**
    * @return double
    */
   public double getSecond() 
   {
	return second;    
   }
   
   /**
    * @param s
    */
   public void setSecond(double s) 
   {
	second = s;    
   }
   
   /**
    * @param p
    * @return int
    */
   public int compareTo(Object p) 
   {
	return getFirst().compareTo(((CouplePF)p).getFirst());    
   }
   
   /**
    * @return String
    */
   public final String toString() 
   {
	return new String("["+getFirst()+";"+getSecond()+"]");    
   }
}
