
/**
* MACS is an Anticipatory Learning Classifier System written in Java.
* 
* Copyright (C) Pierre Grard and Olivier Sigaud with the occasional help
* of Fabien Flacher.
* Contact Olivier.Sigaud@lip6.fr for any queries, or write to
* 
* Olivier Sigaud
* AnimatLab
* Laboratoire d'Informatique de Paris 6
* 8, rue du Capitaine Scott,
* 75015 PARIS, FRANCE
* 
* This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
* 
* More informations about the GNU General Public License can be found
* at the following web site:
* http://www.opensource.org/licenses/gpl-license.php
**/

package MACS;

import java.util.*;
import Utils.StaticRandom;
import Classifier.Perception;
import Classifier.Action;
import Classifier.Token;

/**
 * The ExploMemory class implements a memory of all actions performed
 * in all known situation.
 * It is organized as a Map whose key is a Perception and whose value
 * is an ActionTable.
* @author <a href="mailto:Olivier.Sigaud@lip6.fr"> Olivier Sigaud</a> and <a href="mailto:gerpy@free.fr"> Pierre Grard</a>.
* 
 */
final class ExploMemory extends TreeMap 
{
    public static boolean traceFlag = false;//true;//

   public ExploMemory()
   {
   }
   
   /**
    * @param p
    * @return MACS.ActionTable
    */
   public ActionTable getActionTable(Perception p) 
   {
	return (ActionTable)get(p);    
   }
   
   /**
    * @param p
    * @param nbActions
    */
   public void add(Perception p, int nbActions) 
   {
       //if (traceFlag) System.out.println("Appel de add : "+p+super.toString());
	super.put(p,new ActionTable(nbActions));    
   }
   
   /**
    * @param p
    * @param index
    */
   public void addAction(Perception p, Action ac) 
   {
       byte  index = ac.getToken((byte)0).getValue();
       //if (traceFlag) System.out.println("Appel de addAction : "+p+super.toString());
	((ActionTable)get(p)).addAction(index);
   }
   
   /**
    * @param p
    * @return int 
    */
   public int getBooleanImmediateValue(Perception p) 
   {
	return ((ActionTable)get(p)).getBooleanReward();    
   }
   
   /**
    * @param p
    * @return int 
    */
   public int getPreciseImmediateValue(Perception p) 
   {
	return ((ActionTable)get(p)).getPreciseReward();    
   }
}
