
/**
* MACS is an Anticipatory Learning Classifier System written in Java.
* 
* Copyright (C) Pierre Grard and Olivier Sigaud with the occasional help
* of Fabien Flacher.
* Contact Olivier.Sigaud@lip6.fr for any queries, or write to
* 
* Olivier Sigaud
* AnimatLab
* Laboratoire d'Informatique de Paris 6
* 8, rue du Capitaine Scott,
* 75015 PARIS, FRANCE
* 
* This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
* 
* More informations about the GNU General Public License can be found
* at the following web site:
* http://www.opensource.org/licenses/gpl-license.php
**/

package MACS;

import java.util.*;
import Utils.StaticRandom;
import Utils.ValuesDistribution;
import Classifier.Perception;
import Classifier.Action;

/**
 * The HierarchicalAgregator class is in charge of choosing the next action
 * depending on several models of payoff.
* @author <a href="mailto:Olivier.Sigaud@lip6.fr"> Olivier Sigaud</a> and <a href="mailto:gerpy@free.fr"> Pierre Grard</a>.
* 
 */
final class HierarchicalAgregator
{
    public static boolean DEBUG = false;// true;//
   private static final double VALUES_ZERO_THRESHOLD = 0.0000000001;// 10E-10
private	int nbPossibilities;

    private double values[][];
    public HierarchicalAgregator(int nbpossib)
    {
	nbPossibilities = nbpossib;
	values = new double[ClassifierSystemParameters.nbCriteria][nbpossib];
    }
    /**
     * @param index
     * @param hierarchyLevel
     * @return double
     */

    public double getValue(int hierarchyLevel, int index) 
    {
	return values[hierarchyLevel][index];
    }
    public void setValue(int hierarchyLevel, int index, double value)
    {
	if (value<VALUES_ZERO_THRESHOLD) values[hierarchyLevel][index] = 0.0;
	else values[hierarchyLevel][index] = value;
    }
   
    /**
     * This methods finds the number of an anticipated situation that has the required value 
     * according to the selection mode
     * @param value : the value that the anticipated situation must have
     * @param hierarchylevel : depends on the criteria order.
     * For instance : 0 = information, 1 = external reward, 2 = rehearsal
     * @return int : the index of the anticipated situation found
     */
    public int pickPossibWithValue(double value, int hierarchylevel) 
    {
	ValuesDistribution distrib = new ValuesDistribution((short)nbPossibilities);
	for (int i=0; i<nbPossibilities ; i++)
	    {
		distrib.add(getValue(hierarchylevel,i));
	    }
	return distrib.getRandomIndexOfValueEqualTo(value);
    }
}
