
/**
* MACS is an Anticipatory Learning Classifier System written in Java.
* 
* Copyright (C) Pierre Grard and Olivier Sigaud with the occasional help
* of Fabien Flacher.
* Contact Olivier.Sigaud@lip6.fr for any queries, or write to
* 
* Olivier Sigaud
* AnimatLab
* Laboratoire d'Informatique de Paris 6
* 8, rue du Capitaine Scott,
* 75015 PARIS, FRANCE
* 
* This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
* 
* More informations about the GNU General Public License can be found
* at the following web site:
* http://www.opensource.org/licenses/gpl-license.php
**/

package MACS;

import java.util.*;
import Classifier.Action;
import Classifier.Perception;

/**
 * The InfoValueEstimator class is in charge of returning
 * the estimated immediate information value in the current
 * situation from the informations provided by the Integrator.
* @author <a href="mailto:Olivier.Sigaud@lip6.fr"> Olivier Sigaud</a> and <a href="mailto:gerpy@free.fr"> Pierre Grard</a>.
* 
 */

final class InfoValueEstimator
{
    public static boolean DEBUG = false;// true;//
    private double immediateInfoGains[];
    protected ClassifierSystem modelOfTransitions;


    public InfoValueEstimator(ClassifierSystem c)
    {
	modelOfTransitions = c;
    }

    /**
     * @param possib
     */

    protected final void fillImmediateInfoValue(Vector[] possib)
    {
	immediateInfoGains = new double[possib[0].size()];
	Vector vAnticipations = possib[1];
	Vector vInfoGains = possib[2];
	if (DEBUG)
	    {
		System.out.println("****************************************************************************");
		System.out.println("fillImmediateInfoValue : before :");
		System.out.println("vActions :" + possib[0]);
		System.out.println("vInfoGains :" + vInfoGains);
		for (int i=0 ; i<vAnticipations.size() ; i++)
		    {
			System.out.println("anticipations["+i+"] : " + ((Perception)vAnticipations.get(i)));
		    }
	    }
	for (int i=0 ; i<vAnticipations.size() ; i++)
	    {
		if (((Perception)vAnticipations.get(i))!=null)
		    {
			immediateInfoGains[i] = ((Double)vInfoGains.get(i)).doubleValue();
		    }
		else
		    {
			immediateInfoGains[i] = 1.0;
		    }
	    }
	maximizeImmediateInfoGain(possib[0]);
    }

    /**
     * This method computes the ImmediateInfoGain for each possible anticipated 
     * situation, given that for the same action different values can be estimated:
     * for each situation and for a given action,
     * it takes the max of the values computed for each situation resulting from this action
     * for each (S0,a,S1) triple
     * for all those whose action is a
     * their ImmediateInfoGain is the max ImmediateInfoGain of the group
     */
    private void maximizeImmediateInfoGain(Vector actions) 
    {
	for (short i=0; i<modelOfTransitions.getNbActions(); i++)
	    {
		Action action = modelOfTransitions.getAction(i);
		double maxGainForAction = Double.NEGATIVE_INFINITY;
		for (int j=0; j<actions.size() ; j++)
		    if ((action.equals((Action)actions.get(j))) && (immediateInfoGains[j]>maxGainForAction))
			maxGainForAction = immediateInfoGains[j];
		for (int j=0; j<actions.size() ; j++)
		    if (action.equals((Action)actions.get(j))) immediateInfoGains[j] = maxGainForAction;
	    }
    }

public double getImmediateInfoGains(int index)
    {
	return immediateInfoGains[index];
    }
}// end of InfoValueEstimator
