
/**
* MACS is an Anticipatory Learning Classifier System written in Java.
* 
* Copyright (C) Pierre Grard and Olivier Sigaud with the occasional help
* of Fabien Flacher.
* Contact Olivier.Sigaud@lip6.fr for any queries, or write to
* 
* Olivier Sigaud
* AnimatLab
* Laboratoire d'Informatique de Paris 6
* 8, rue du Capitaine Scott,
* 75015 PARIS, FRANCE
* 
* This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
* 
* More informations about the GNU General Public License can be found
* at the following web site:
* http://www.opensource.org/licenses/gpl-license.php
**/

package MACS;

import java.util.*;

import Utils.StaticRandom;
import Classifier.Perception;

/**
 * The RandomPlanner class defines the random choice of planning steps.
* @author <a href="mailto:Olivier.Sigaud@lip6.fr"> Olivier Sigaud</a> and <a href="mailto:gerpy@free.fr"> Pierre Grard</a>.
* 
 */
class RandomPlanner extends AbstractPlanner 
{
    public RandomPlanner(MACSActionSelector s, int nb)
    {
	super(s,nb);
    }
    /**
     */
    public final void updatePolicy(Anticipator anticipator)
    {
	PerceptionList perceptions = selector.getPerceptionList();
	int range = perceptions.size();
	//if (DEBUG) System.out.println("updatePolicy Nb planning steps : " + nbplanningsteps + ":" + perceptions);
	for (int i=0 ; i<nbPlanningSteps; i++)
	    {
		Iterator it = perceptions.iterator();
		int choice = StaticRandom.nextInt(range);
		for (int j=0;j<choice;j++) it.next();
		Perception p = (Perception)it.next();
		if (!selector.getModelOfPayoff().isFinal(p)) 
		    {
			Vector possib[] = anticipator.getPossibilityTables(p);		
			selector.selectAction(null, p, false, possib);
		    }
	    }    
    }
}
