
/**
* MACS is an Anticipatory Learning Classifier System written in Java.
* 
* Copyright (C) Pierre Grard and Olivier Sigaud with the occasional help
* of Fabien Flacher.
* Contact Olivier.Sigaud@lip6.fr for any queries, or write to
* 
* Olivier Sigaud
* AnimatLab
* Laboratoire d'Informatique de Paris 6
* 8, rue du Capitaine Scott,
* 75015 PARIS, FRANCE
* 
* This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
* 
* More informations about the GNU General Public License can be found
* at the following web site:
* http://www.opensource.org/licenses/gpl-license.php
**/

package MACS;

import java.util.*;
import java.lang.reflect.Array;
import Utils.ValuesDistribution;
import Classifier.AnticipatoryClassifier;
import Classifier.Perception;
import Classifier.Action;
import Classifier.Anticipation;
import Classifier.MACSAnticipation;
import Classifier.YACSAnticipation;

/**
 * The SpecializationManager class is in charge of specializing classifiers in the 
 * model of transitions.
 * The only method called from outside is run().
* @author <a href="mailto:Olivier.Sigaud@lip6.fr"> Olivier Sigaud</a> and <a href="mailto:gerpy@free.fr"> Pierre Grard</a>.
* 
 */
final class SpecializationManager 
{
   static final boolean DEBUG = false;
   private ClassifierSystem classifierSystem;
   
   /**
    * @param cs
    */
   public SpecializationManager(ClassifierSystem cs) 
   {
	classifierSystem = cs;    
   }
   
   /**
    * @return MACS.PerceptionList
    */
   protected PerceptionList getPerceptionList() 
   {
	return classifierSystem.getPerceptionList();    
   }
   
   /**
    * @param lastPerception
    * @param lastAction
    */
   public void run(Perception lastPerception, Action lastAction) 
   {
	for (short i=0; i<classifierSystem.getNbClassifiers() ; i++)
	    {
		AnticipatoryClassifier classifier = (AnticipatoryClassifier)classifierSystem.getClassifier(i);
		if (classifier.doesOscillate(ClassifierSystemParameters.nbEvaluationsToSpecialize) && (!classifier.getCondition().isSpecialized()))
		    {
			if (ClassifierSystemParameters.traceMutspecRemoval) 
			    classifierSystem.getTraceManager().getClassifiersRemovedByMutSpec().addClassifier(classifier, false);
			classifierSystem.removeClassifier(classifier);
			i--;

			ValuesDistribution vd = classifier.buildValuesDistributionForReliabilities();
			for (byte j=0 ; j<vd.getNbValues() ; j++)
			    if (!classifier.getCondition().isDontCare(j)) vd.setValue(j, Double.POSITIVE_INFINITY);
			byte indexToMutSpec = (byte)(vd.computeIndexMin());
			AnticipatoryClassifier newClassifier = classifier.computeGoodSpecialized(indexToMutSpec,ClassifierSystemParameters.initialReliability,ClassifierSystemParameters.learningRate);

			if (ClassifierSystemParameters.traceMutspecCreation)
			    classifierSystem.getTraceManager().getClassifiersCreatedByMutSpec().addClassifier(newClassifier, true);
			classifierSystem.addClassifier(newClassifier, true);
		    }
	    }    
   }
}
