
/**
* MACS is an Anticipatory Learning Classifier System written in Java.
* 
* Copyright (C) Pierre Grard and Olivier Sigaud with the occasional help
* of Fabien Flacher.
* Contact Olivier.Sigaud@lip6.fr for any queries, or write to
* 
* Olivier Sigaud
* AnimatLab
* Laboratoire d'Informatique de Paris 6
* 8, rue du Capitaine Scott,
* 75015 PARIS, FRANCE
* 
* This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
* 
* More informations about the GNU General Public License can be found
* at the following web site:
* http://www.opensource.org/licenses/gpl-license.php
**/
package MACS;

import java.util.*;

import Utils.StaticRandom;
import Classifier.Perception;

/**
 * The SystematicPlanner class defines the systematic choice of planning steps.
 * One step corresponds to one complete value iteration sweep.
* @author <a href="mailto:Olivier.Sigaud@lip6.fr"> Olivier Sigaud</a> and <a href="mailto:gerpy@free.fr"> Pierre Grard</a>.
* 
 */
class SystematicPlanner extends AbstractPlanner 
{
    protected static final double EVALUATION_RATIO = 1.0;

    public SystematicPlanner(MACSActionSelector s, int nb)
    {
	super(s,nb);
    }
   
    /**
     * @return int the number of fake actions (planning steps) performed between two actual actions
     */
    protected int getNbFakes() 
    {
	return (int)(EVALUATION_RATIO*selector.getPerceptionList().size());
    }
   
    /**
     */
     public final void updatePolicy(Anticipator anticipator)
    {
	PerceptionList perceptions = selector.getPerceptionList();
	Iterator it = perceptions.iterator();
	Perception p = null;
	//if (DEBUG) System.out.println("updatePolicy Nb fakes : " + nbFakes + ":" + perceptions);
	for (int i=0 ; i<getNbFakes(); i++)
	    {
		if (it.hasNext())
		    p = (Perception)it.next();
		else
		    { 
			it = perceptions.iterator();
			p = (Perception)it.next();
		    }
		if (!selector.getModelOfPayoff().isFinal(p))
		    {
			Vector possib[] = anticipator.getPossibilityTables(p);		
			selector.selectAction(null, p, false, possib);
		    }
	    }    
    }
}
