
/**
* MACS is an Anticipatory Learning Classifier System written in Java.
* 
* Copyright (C) Pierre Grard and Olivier Sigaud with the occasional help
* of Fabien Flacher.
* Contact Olivier.Sigaud@lip6.fr for any queries, or write to
* 
* Olivier Sigaud
* AnimatLab
* Laboratoire d'Informatique de Paris 6
* 8, rue du Capitaine Scott,
* 75015 PARIS, FRANCE
* 
* This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
* 
* More informations about the GNU General Public License can be found
* at the following web site:
* http://www.opensource.org/licenses/gpl-license.php
**/

package Utils;

import java.io.*;
import java.awt.*;
import java.util.*;
import java.awt.event.*;
import java.lang.reflect.*;

/**
 * Class that helps debugging, indenting all the output traces
 * <P>
 * @author Fabien Flacher (/08/01)
 * @version $Revision: 1.1.1.1 $
 */


public class Debugger
{

    /**
    * Hashtable used by of the Debugger
    */
    protected static Hashtable myHashtable ;
    /**
    * Set the Hashtable used by of the Debugger
    */
    private static final void setHashtable (Hashtable hashtable )
    {
        myHashtable = hashtable ;
    }
    /**
    * Get the Hashtable used by of the Debugger 
    */
    public static final Hashtable getHashtable ()
    {
        return myHashtable ;
    }

    /**
     * Indentation value of the Debugger
     */
    protected static int myIndent;
    /**
     * Set the Indentation value of the Debugger
     */  
    private static final void setIndent(int myI) 
    {
	myIndent = myI;
    }
    /**
     * Get the Indentation value of the Debugger
     */  
    public static final int getIndent() 
    {
	return myIndent;
    }

//----------------------------------------------------------------------//

    /**
     * Construct the Debugger
     * @param myI Indentation value of the Debugger
     */  
    public Debugger(int myI) 
    {
	setIndent(myI);
    }

    /**
     * Add 1 to the Indentation value of the Debugger
     */  
    public static final void addIndent() 
    {
	//Exception e = new Exception(); e.printStackTrace();
	setIndent(getIndent()+1);
    }
    /**
     * Sub 1 the Indentation value of the Debugger
     */  
    public static final void subIndent() 
    {
	setIndent(getIndent()-1);
    }

    /**
     * Return a new line with the indentation value of the Debugger
     */  
    public static final String newLine() 
    {
	String s="\n";
	for(int i=0; i<getIndent(); i++)
	    s += "\t";
	return s;
    }

    /**
     * Return a String with the indentation value of the Debugger
     */  
    public static final String newTab() 
    {
	String s="";
	for(int i=0; i<getIndent(); i++)
	    s += "\t";
	return s;
    }

    /**
     * Return a String with the indentation value of the Debugger
     */  
    public static final String newTabFirst() 
    {
	init();
	String s="";
	for(int i=0; i<getIndent(); i++)
	    s += "\t";
	return s;
    }

    /**
     * Return a String describing the object contained by the Hashtable
     */  
    public static final String printHashtableComponentNames(Hashtable hashtable) 
    {
	String s = "";
	for (Enumeration e = hashtable.keys() ; e.hasMoreElements() ;) 
	    {
		Object nKey = e.nextElement();
		Object nElt = hashtable.get(nKey);
		String name = nElt.getClass().getName();
		String addS = name;
		if (name.equals("java.util.Vector"))
		    addS = "[" +printVectorComponentNames(((Vector)nElt)) +"]";
		if (name.equals("java.util.Hashtable"))
		    addS = "[" +printHashtableComponentNames(((Hashtable)nElt)) +"]";
		s += " " +nKey.getClass().getName() +"@" +nKey.hashCode() +" ->" +addS;
	    }
	return s;
    }

     /**
     * Return a String describing the object contained by the Hashtable
     */  
    public static final String printHashtableArrayVals(Hashtable[] hashtable) 
    {
	//System.out.print(Debugger.newTab() +"Debugger::printHashArrayVal -hash:" +hashtable);System.out.println (" -taille:" +hashtable.length);
	String s = "";
	for(int i=0; i<hashtable.length; i++)
	    {
		s += "\n[" +i +":";
		if (hashtable[i] != null)
		    for (Enumeration e = hashtable[i].keys() ; e.hasMoreElements() ;) 
			{
			    Object nKey = e.nextElement();
			    Object nElt = hashtable[i].get(nKey);
			    s += " #" +nKey  +"~" +nElt;
			}
		else
		    s += "null";
		s += "]";
	    }
	return s;
    }

   /**
     * Return a String describing the object contained by the Hashtable
     */  
    public static final String printVectorComponentNames(Vector vector) 
    {
	String s = "";
	for (Enumeration e = vector.elements() ; e.hasMoreElements() ;) 
	    {
		Object nElt = e.nextElement();
		String name = nElt.getClass().getName();
		String addS = name;
		if (name.equals("java.util.Vector"))
		    addS = "[" +printVectorComponentNames(((Vector)nElt)) +"]";
		if (name.equals("java.util.Hashtable"))
		    addS = "[" +printHashtableComponentNames(((Hashtable)nElt)) +"]";
		s +=  " " +addS;
	    }
	return s;
    }

    /**
     * Return a String describing String Array
     */  
    public static final String printStringArray(String[] stringA) 
    {
	String s = "[";
	for (int i=0; i<stringA.length; i++)
	    s += " " +stringA[i];
	return s +"]";
    }

    /**
     * Initialize the indentation value of the Debugger
     */  
    public static final void init() 
    {
	myIndent = 0;
    }

    /**
     * f the Debugger
     */  
    public static final void addCall(String packageName, String className, String methodName) 
    {
	if (getHashtable() == null)
	    setHashtable(new Hashtable());
	if (!getHashtable().containsKey(packageName))
	    {
		Hashtable classList = new Hashtable();
		Hashtable methodList = new Hashtable();
		Integer numerosity  = new Integer(0);
		methodList.put(methodName, numerosity);
		classList.put(className, methodList);
		getHashtable().put(packageName, classList);
	    }
	else
	    {
		Hashtable classList = ((Hashtable)(getHashtable().get(packageName)));
		if (!classList.containsKey(className))
		    {
			Hashtable methodList = new Hashtable();
			Integer numerosity  = new Integer(0);
			methodList.put(methodName, numerosity);
			classList.put(className, methodList);
		    }
		else
		    {
			Hashtable methodList = ((Hashtable)(classList.get(className)));
			if (!methodList.containsKey(methodName))
			    {
				Integer numerosity  = new Integer(0);
				methodList.put(methodName, numerosity);
			    }
			else
			    {
				Integer numerosity = ((Integer)(methodList.get(methodName)));
				numerosity = new Integer(numerosity.intValue() +1);
				methodList.put(methodName, numerosity);
			    }
		    }
	    }
    }

    /**
     * Debugger
     */  
    public static void saveNumerosityCall()
    {
	if (getHashtable() != null)
	    {
		FilesWriter writer = new FilesWriter("numerosity.dat");
		writer.open();
		for (Enumeration e1 = getHashtable().keys() ; e1.hasMoreElements() ;) 
		    {
			String packageName= ((String)e1.nextElement());
			Hashtable classList = ((Hashtable)(getHashtable().get(packageName)));
			for (Enumeration e2 = classList.keys() ; e2.hasMoreElements() ;) 
			    {
				String className= ((String)e2.nextElement());
				Hashtable methodList = ((Hashtable)(classList.get(className)));
				for (Enumeration e3 = methodList.keys() ; e3.hasMoreElements() ;) 
				    {
					String methodName= ((String)e3.nextElement());
					Integer numerosity = ((Integer)(methodList.get(methodName)));
					writer.outPrintln(numerosity.intValue() +"\t" +packageName +"\t" +className +"\t" +methodName);
				    }
			    }
		    }
		writer.close();
		System.out.println("Numerosity file writed");
	    }
	else
	    System.out.println("No numerosity data computed");
    }

    /**
     * A test routine
     */  
    public static void main(String[] args)
    {
    }
    
}
