
/**
* MACS is an Anticipatory Learning Classifier System written in Java.
* 
* Copyright (C) Pierre Grard and Olivier Sigaud with the occasional help
* of Fabien Flacher.
* Contact Olivier.Sigaud@lip6.fr for any queries, or write to
* 
* Olivier Sigaud
* AnimatLab
* Laboratoire d'Informatique de Paris 6
* 8, rue du Capitaine Scott,
* 75015 PARIS, FRANCE
* 
* This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
* 
* More informations about the GNU General Public License can be found
* at the following web site:
* http://www.opensource.org/licenses/gpl-license.php
**/

package Utils;

import java.io.*;
import java.awt.*;
import java.util.*;
import java.awt.event.*;
import java.lang.reflect.*;

/**
 * Class that defines a container of Strings indexed by their creation rank.
 * You can get a string by its number or the number by its String.
 * <P>
 * @author <a href="http://animatlab.lip6.fr/Sigaud">Olivier Sigaud</a>
 * @version $Revision: 1.1.1.1 $
 */


public class IndexedStringTable
{

    /**
     * List of instances of the IndexedStringTable
     */
    protected Vector instances = new Vector();
    /**
     * List of instances of the IndexedStringTable
     */
    protected Hashtable myListOfInstance = new Hashtable();
    /**
     * List of instances of the IndexedStringTable
     */
    protected Hashtable myInstanceCounter = new Hashtable();
    /**
     * Set the list of instances of the IndexedStringTable
     */
    private final void setListOfInstances(Hashtable myLOI)
    {
	myListOfInstance = myLOI;
    }
    /**
     * Get the list of instances of the IndexedStringTable
     */
    private final Hashtable getListOfInstances()
    {
	return myListOfInstance;
    }
    /**
     * Checks whether the String is already present within the instances
     */

    private boolean contains(String str)
    {
	return (myListOfInstance.containsKey(str));
    }
    /**
     * Add a new String in the IndexedStringTable
     */
    private final void addInstance(String str)
    {
	Integer i = new Integer(nbInstances);
	myListOfInstance.put(str,i);
	nbInstances++;
	instances.add(str);
    }

    /**
     * Nb maximum  of different Strings
     */
    protected int nbInstances = 0;
    /**
     * Get the nb of different Strings
     */
    public final int getNbInstances()
    {
	return nbInstances;
    }
    /**
     * Get the nb of instances of one particular String
     */
    public final int getNbInstances(String str)
    {
	Integer nb = (Integer)myInstanceCounter.get(str);
	return nb.intValue();
    }
    /**
     * Get one String from its number
     * @return the String if the number is correct, null otherwise
     */
    public final String getStringFromNumber(int index)
    {
	if (index<nbInstances)	return (String)(instances.elementAt(index));
	else 
	    {
		System.out.println("IndexedStringTable:getStringFromNumber : Index out of bound !");
		return null;
	    }
    }
    /**
     * Get the number of one String
     * @return  the number of the String if it exists, -1 otherwise
     */
    public final int getNumberFromString(String str)
    {
	Integer num = (Integer)(myListOfInstance.get(str));
	if (num!=null)	return num.intValue();
	else return -1;
    }

    /**
     * Add a new String in the IndexedStringTable
     * if its not already present.
     * @param str is the String to add
     * @param val is true if the String should be counted as a new instance
     */

    public void addString(String str, boolean val)
    {
	if (!contains(str))
	    {
		addInstance(str);
		if (val) myInstanceCounter.put(str,new Integer(1));
		else myInstanceCounter.put(str,new Integer(0));
	    }
	else if (val)
	    {
		int cpt = getNbInstances(str);
		myInstanceCounter.put(str,new Integer(cpt+1));
	    }
    }

    /**
     * Return a String describing the RelevantPointType instance
     */
    public String toString()
    {
	String s = "";
	Enumeration enum = myListOfInstance.keys();
	Enumeration enum2 = myListOfInstance.elements();
	while(enum.hasMoreElements())
	    {
		s += "|" +enum.nextElement() +"|="+enum2.nextElement() +", ";
	    }
	return s;
    }

    /**
     * A test routine
     */
    public static void main(String[] args)
    {
	Chrono c = new Chrono();
	IndexedStringTable t = new IndexedStringTable();
	for (int i=0;i<5;++i)
	    {
		String toto = "coucou" + i;
		String toto2 = "coucou";
		t.addString(toto,true);
		t.addString(toto2,true);
		System.out.println("contenu :" + t.getNbInstances());
		System.out.println("index de :" + toto + " : " + t.getNumberFromString(toto));
		System.out.println("index de :" + toto2 + " : " + t.getNumberFromString(toto2));
		System.out.println("contenu de rang :" + i + " : " + t.getStringFromNumber(i));
	    }
	System.out.println("contenu global :" + t);
	c.stop();
    }
}
