
/**
* MACS is an Anticipatory Learning Classifier System written in Java.
* 
* Copyright (C) Pierre Grard and Olivier Sigaud with the occasional help
* of Fabien Flacher.
* Contact Olivier.Sigaud@lip6.fr for any queries, or write to
* 
* Olivier Sigaud
* AnimatLab
* Laboratoire d'Informatique de Paris 6
* 8, rue du Capitaine Scott,
* 75015 PARIS, FRANCE
* 
* This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
* 
* More informations about the GNU General Public License can be found
* at the following web site:
* http://www.opensource.org/licenses/gpl-license.php
**/

package Utils;

import java.util.Hashtable;
import java.util.Enumeration;
import java.io.FileReader;
import java.io.BufferedReader;
/**
 *This class is used to store a set of parameters.
 * @author <a href="mailto:Olivier.Sigaud@lip6.fr"> Olivier Sigaud</a> and <a href="mailto:gerpy@free.fr"> Pierre Grard</a>.
 */
public final class ParameterSet
{
    protected static Hashtable hash;

    public ParameterSet (String fileName)
    {
	hash = new Hashtable(); 
	addParametersFromFile(fileName);
    }

    public final void addParametersFromFile(String fileName)
    {
	try
	    {
		String line = new String();
		FileReader fileReader = new FileReader(fileName);
		BufferedReader buffReader = new BufferedReader(fileReader);
		while (buffReader.ready())
		    {
			line = buffReader.readLine();
			addParameter(line);
		    }
		fileReader.close();
	    }
	catch (Exception e) {e.printStackTrace();System.exit(-1);}
    }

    private final void addParameter(String line)
    {
	line = Misc.removeSpaces(line);
	if (line.lastIndexOf("//")!=-1) line = line.substring(0, line.lastIndexOf("//"));
	if ((!line.equals("")) && (!line.startsWith("//")) && (line.indexOf('=')!=-1))
	    {
		String param = line.substring(0, line.indexOf('='));
		String value = line.substring(line.indexOf('=')+1);
		hash.put(param, value);
	    }
    }

    public final String getValue(String param)
    {
	String value = (String)hash.get(param);

	if (value==null)
	    System.err.println("ERROR in Utilities.ParameterSet::getValue() : I was unable to load parameter " + param);

	return value;
    }

    public final String getValue(String param, int index)
    {
	String completeparam = param + index;
	String value = (String)hash.get(completeparam);

	if (value==null)
	    System.err.println("ERROR in Utilities.ParameterSet::getValue() : I was unable to load parameter " + completeparam);

	return value;
    }

    public final String toString()
    {
	String strTemp = "";

	Enumeration enumKeys = hash.keys();
	Enumeration enumElts = hash.elements();

	for (short i=0; i<hash.size(); i++)
	    strTemp = strTemp + (String)(enumKeys.nextElement()) + " : " + (String)(enumElts.nextElement()) + "\n";

	return (strTemp);
    }
}
