
/**
* MACS is an Anticipatory Learning Classifier System written in Java.
* 
* Copyright (C) Pierre Grard and Olivier Sigaud with the occasional help
* of Fabien Flacher.
* Contact Olivier.Sigaud@lip6.fr for any queries, or write to
* 
* Olivier Sigaud
* AnimatLab
* Laboratoire d'Informatique de Paris 6
* 8, rue du Capitaine Scott,
* 75015 PARIS, FRANCE
* 
* This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
* 
* More informations about the GNU General Public License can be found
* at the following web site:
* http://www.opensource.org/licenses/gpl-license.php
**/

package Utils;

import java.io.FileReader;
import java.io.BufferedReader;
import java.util.Hashtable;
import java.util.Enumeration;

/**
 * Class that may be used to retrieve parameters from a file.
 * Each line of this file must set a parameter.
 * A line takes the form : "parameter = value // comments".
 * <P>
 * @author Pierre Grard
 * @version $Revision: 1.1.1.1 $
 */

public final class ParamsCollector
{
    /**
     * Hastable used by the ParamsCollector
     */  
    public static Hashtable hash;

    private static String parametersFile;


//----------------------------------------------------------------------//

    /**
     * Init method to fill the Hashtable with every file's values
     */  
    public static Hashtable fillHashTable(String fileName)
    {
	parametersFile = fileName;
	hash = new Hashtable(); 
	try
	    {
		String line = new String();
		FileReader fileReader = new FileReader(fileName);
		BufferedReader buffReader = new BufferedReader(fileReader);
		while (buffReader.ready())
		    {
			line = buffReader.readLine();
			addParameter(line);
		    }
		fileReader.close();
	    }
	catch (Exception e) 
	    {
		e.printStackTrace();
		System.out.println (e.toString() + "\n");
		System.exit(-1);
	    }

	return hash;
    }

    private static void addParameter(String line)
    {
	//System.out.println("ParamsCollector::addParameters -" +line);
	line = Utilities.removeSpaces(line);
	if (line.lastIndexOf("//")!=-1) line = line.substring(0, line.lastIndexOf("//"));
	if ((!line.equals("")) && (!line.startsWith("//")) && (line.indexOf('=')!=-1))
	    {
		String param = line.substring(0, line.indexOf('='));
		String value = line.substring(line.indexOf('=')+1);
		//System.out.println("+" + param + "+" + value + "+");
		hash.put(param, value);
	    }
    }

    /**
     * @param param The name of a parameter
     * @return The value of the parameter (if it does not exist, null is returned)
     **/
    public static String getValue(String param)
    {
	String value = (String)hash.get(param);
	return value;
    }

    /**
     * Return a string representation of all parameters and associated values
     **/
    public static String convertIntoString()
    {
	String strTemp = "";

	Enumeration enumKeys = hash.keys();
	Enumeration enumElts = hash.elements();

	for (short i=0; i<hash.size(); i++)
	    strTemp = strTemp + (String)(enumKeys.nextElement()) + " : " + (String)(enumElts.nextElement()) + "\n";

	return (strTemp);
    }

    /**
     * A test routine
     **/
    public static void main(String[] args)
    {
	ParamsCollector.hash  = fillHashTable(args[0]);
	System.out.println(ParamsCollector.convertIntoString());
    }
}
