
/**
* MACS is an Anticipatory Learning Classifier System written in Java.
* 
* Copyright (C) Pierre Grard and Olivier Sigaud with the occasional help
* of Fabien Flacher.
* Contact Olivier.Sigaud@lip6.fr for any queries, or write to
* 
* Olivier Sigaud
* AnimatLab
* Laboratoire d'Informatique de Paris 6
* 8, rue du Capitaine Scott,
* 75015 PARIS, FRANCE
* 
* This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
* 
* More informations about the GNU General Public License can be found
* at the following web site:
* http://www.opensource.org/licenses/gpl-license.php
**/

package Utils;

import java.awt.Color;

import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.Random;
/**
 * The name tells what that class is...
 * It could probably be merged with Misc...
 * @author <a href="mailto:Olivier.Sigaud@lip6.fr"> Olivier Sigaud</a> and <a href="mailto:gerpy@free.fr"> Pierre Grard</a>.
 */	
public class Utilities
{	

    public static byte computeNbTokens(String msgForm)
    {
	byte nbTokens = (byte)1;

	for (byte i=0; i < msgForm.length(); i++)
	    if (msgForm.charAt(i) == '|')
		nbTokens++;

	return (nbTokens);
    }

    public static byte computeNbTokens(String msgForm, byte[] nbValuesArray)
    {
	byte nbTokens = computeNbTokens(msgForm);	
	readNbValues (msgForm, nbValuesArray);

	return (nbTokens);
    }

    public static void readNbValues (String msgForm, byte[] nbValuesArray)
    {
	StringTokenizer tokenizer = new StringTokenizer (msgForm, "|");
	String strTemp;

	byte i = 0;
	do
	    {
		strTemp = tokenizer.nextToken();
		if (i==0)
		    strTemp = strTemp.substring(1);
		if (!tokenizer.hasMoreElements())
		    strTemp = strTemp.substring(0, strTemp.length()-1);
		nbValuesArray[i] = Byte.valueOf(strTemp).byteValue();
		i++;
	    }
	while (tokenizer.hasMoreElements());
    }

    public static long computeSeed(String strSeed)
    {
	System.out.println("Passage par computedSeed (danger, graine non maitrisee !)");
 	if (strSeed.equals("none"))
 	    return (long)(new Random().nextDouble() * (Byte.MAX_VALUE*2+1));
 	else
 	    return Long.valueOf(strSeed).longValue();
    }

    public static boolean convertSwitch(String str)
    {
	if (str.equals("on"))
	    return true;
	else
	    return false;
    }

    public static String putSideBySide (String str1, String str2, short offset)
    {
	String str = new String();
	str1 = str1.trim(); str2 = str2.trim();
	StringTokenizer strTok1 = new StringTokenizer(str1, "\n");
	StringTokenizer strTok2 = new StringTokenizer(str2, "\n");

	while (strTok1.hasMoreTokens() || strTok2.hasMoreTokens())
	    {
		String tok1 = "";
		String tok2 = "";
		if (strTok1.hasMoreTokens()) tok1=strTok1.nextToken();
		if (strTok2.hasMoreTokens()) tok2=strTok2.nextToken();

		str = str
		    + tok1 + buildStringWithSpaces((short)(offset-tok1.length()))
		    + tok2 + buildStringWithSpaces((short)(offset-tok2.length()))
		    + "\n";
	    }

	return str;
    }

    private static String buildStringWithSpaces(short length)
    {
	String str = new String();

	for (short i=0 ; i<length ; i++)
		str = str + " ";

	return str;
    }

    public static String removeSpaces(String string)
    {
	String strTemp = new	String();

	for (short i=0; i<string.length(); i++)
	    if ((string.charAt(i)!=' ') && ((string.charAt(i)!='\t')) )
		strTemp = strTemp + string.charAt(i);

	return (strTemp);
    }

    public static String strReplace(String str, String str1, String str2)
    {
	int index = str.indexOf(str1);
	if (index==-1)
	    return str;
	else
	    {
		String strResult = str.substring(0, index) + str2 + str.substring(index+str1.length());
		return strReplace (strResult, str1, str2);
	    }
    }

    public static Color getColorFromRGB(String descS)
    {
	StringTokenizer st = new StringTokenizer(descS,",");
	if (st.nextToken().equals("RGB"))
	    {
		int r = Integer.parseInt(st.nextToken());
		int g = Integer.parseInt(st.nextToken());
		int b = Integer.parseInt(st.nextToken());
		return new Color(r,g,b);
	    }
	else
	    {
		System.out.println(Debugger.newTabFirst() +"Utilities::getColorFromRGB (bad definition of the color, TYP missing (ex RGB,...)");
		System.exit(1);
	    }
	return new Color(0,0,0);
    }
    
    public static String shortName(Object obj)
    {
	int nbBarre = 4;
	StringTokenizer st = new StringTokenizer(obj.getClass().getName(), ".");
	String name = "";
	while (st.hasMoreTokens())
	    name = st.nextToken();
	int taille = name.length();
	if (taille <= 2*nbBarre)
	    return name;
	int cpt = 0;
	for(int i=0; i<taille; i++)
	    if (Character.isUpperCase(name.charAt(i)))
		cpt = i;
	String s1 = "";
	String s2 = "";
	int d1 = 0;
	int l1 = Math.min(nbBarre, cpt);
	int d2 = cpt;
	int l2 = Math.min(nbBarre, taille -cpt);
	boolean touched = false;
	if (l1 != nbBarre)
	    {
		l2 += nbBarre-l1;
		touched = true;
	    }
	if (l2 != nbBarre && !touched)
	    l1 += nbBarre-l2;
	s1 = name.substring(d1, d1 +l1);
	s2 = name.substring(d2, d2 +l2);
	//System.out.println("s1:" +s1 +" s2:" +s2);
	return s1 +s2;
    }

    public static String roundValue(double val)
    {
 	return (String.valueOf((double)Math.round(100000.0*val)/100000.0)+"0000000").substring(0,7);
    }

    public static String roundValue(double val, int nbValAfterDot)
    {
 	return (String.valueOf((double)Math.round(100000.0*val)/100000.0)+"0000000000000000").substring(0,nbValAfterDot+1);
    }

    public static String sValue(double val, int nbValAfterDot)
    {
 	return (String.valueOf(val)+"xxxxxxxxxxxxxxxxxxxxxxxxxxxxx").substring(0,nbValAfterDot+1);
    }

    public static void main(String[] args)
    {
	double res = 0.0;
	double val1 = Double.parseDouble(args[0]);
	double val2 = Double.parseDouble(args[1]);
	System.out.println("val1 = " +val1);
	System.out.println("val2 = " +val2);
	if (args[2].equals("*")) res = val1 * val2;
	if (args[2].equals("/")) res = val1 / val2;
	if (args[2].equals("+")) res = val1 + val2;
	if (args[2].equals("-")) res = val1 - val2;
	System.out.println("res  = " +res +" (val1 " +args[2] +" val2)");
    }
}
