
/**
* MACS is an Anticipatory Learning Classifier System written in Java.
* 
* Copyright (C) Pierre Grard and Olivier Sigaud with the occasional help
* of Fabien Flacher.
* Contact Olivier.Sigaud@lip6.fr for any queries, or write to
* 
* Olivier Sigaud
* AnimatLab
* Laboratoire d'Informatique de Paris 6
* 8, rue du Capitaine Scott,
* 75015 PARIS, FRANCE
* 
* This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
* 
* More informations about the GNU General Public License can be found
* at the following web site:
* http://www.opensource.org/licenses/gpl-license.php
**/
package Utils;

/**
 * A Values Distribution is used to perform various operations on double distributions
 *
 * @author Pierre GERARD (gerpy@free.fr)
**/
public final class ValuesDistribution
{
    double distribArray[]; 
    short size;

    public ValuesDistribution (short nbMaxElements)
    {
	distribArray = new double[nbMaxElements];
	size = 0;
    }

    public ValuesDistribution(double[] values, short nbElements)
    {
	distribArray = new double[nbElements];
	size = nbElements;

	for (short i=0; i<nbElements; i++)
	    distribArray[i] = values[i];
    }

    public ValuesDistribution (int[] values, short nbElements)
    {
	distribArray = new double[nbElements];
	size = nbElements;

	for (short i=0; i<nbElements; i++)
	    distribArray[i] = (double)(values[i]);
    }

    public final void add(double value)
    {
       distribArray[size++] = value;
    }

    public final double getValue(short index)
    {
	return distribArray[index];
    }

    public final short getNbValues()
    {
	return size;
    }

    public final void setValue(short index, double value)
    {
	distribArray[index] = value;
    }

    public final void updateToEqui()
    {
	for (short i=0; i<size; i++)
	    distribArray[i] = (1.0 / (double)size);
    }

    public final boolean containsEqualValues()
    {
	double value1 = distribArray[0];

	for (short i=1; i<size; i++)
	    {
		double value2 = distribArray[1];
		if (value1!=value2)
		    return  false;
		value1 = value2;
	    }

	return true;
    }

    public final short computeIndexFirstMax()
    {
	short iMax = 0;

	for (short i = 1; i<size; i++)
	    if (distribArray[i]>distribArray[iMax])
		iMax=i;

	return iMax;
    }

    public final short computeIndexLastMax()
    {
	short iMax = 0;

	for (short i = 1; i<size; i++)
	    if (distribArray[i]>=distribArray[iMax])
		iMax=i;

	return iMax;
    }

    public final short computeIndexMax()
    {
	short iMax = 0;
	short maxArray[] = new short[size];
	short nbMax = 1;
	maxArray[0] = 0;

	for (short i = 1; i<size; i++)
	    if (distribArray[i]>distribArray[iMax])
		{
		    maxArray[0] = i;
		    iMax = i;
		    nbMax = 1;
		}
	    else if (distribArray[i]==distribArray[iMax])
		{
		    maxArray[nbMax] = i;
		    nbMax++;
		}
	return maxArray[(short)(StaticRandom.nextDouble()*(double)nbMax)];
    }

    public final short    getRandomIndexOfValueEqualTo(double value)
    {
	short maxArray[] = new short[size];
	short nb = 0;
	for (short i = 0; i<size; i++)
	    if (distribArray[i]==value)
		{
		    maxArray[nb++] = i;
		}
	return maxArray[(short)(StaticRandom.nextDouble()*(double)nb)];
    }
    public final short computeIndexAbsoluteMax()
    {
	double vMax = distribArray[0];
	short nbMax = 1;
	short iMax = 0;

	for (short i = 1; i<size; i++)
	    if (distribArray[i]>vMax)
		{
		    vMax = distribArray[i];
		    iMax = i;
		    nbMax = 1;
		}
	    else if (distribArray[i]==vMax)
		nbMax++;

	if (nbMax>1)
	    return -1;
	else
	    return iMax;
    }

    public final short computeIndexAbsoluteMin()
    {
	double vMin = distribArray[0];
	short nbMin = 1;
	short iMin = 0;

	for (short i = 1; i<size; i++)
	    if (distribArray[i]<vMin)
		{
		    vMin = distribArray[i];
		    iMin = i;
		    nbMin = 1;
		}
	    else if (distribArray[i]==vMin)
		nbMin++;

	if (nbMin>1)
	    return -1;
	else
	    return iMin;
    }

     public final short computeIndexFirstMin()
    {
	short iMin = 0;

	for (short i = 1; i<size; i++)
	    if (distribArray[i]<distribArray[iMin])
		iMin=i;

	return iMin;
    }

    public final short computeIndexLastMin()
    {
	short iMin = 0;

	for (short i = 1; i<size; i++)
	    if (distribArray[i]<=distribArray[iMin])
		iMin=i;

	return iMin;
    }

    public final short computeIndexMin()
    {
	short iMin = 0;
	short minArray[] = new short[size];
	short nbMin = 1;
	minArray[0] = 0;

	for (short i = 1; i<size; i++)
	    if (distribArray[i]<distribArray[iMin])
		{
		    minArray[0] = i;
		    iMin = i;
		    nbMin = 1;
		}
	    else if (distribArray[i]==distribArray[iMin])
		{
		    minArray[nbMin] = i;
		    nbMin++;
		}

	return minArray[(short)(StaticRandom.nextDouble()*(double)nbMin)];
    }

    public final short computeIndexFirstMaxAbs()
    {
	short iMax = 0;

	for (short i = 1; i<size; i++)
	    if (Math.abs(distribArray[i])>Math.abs(distribArray[iMax]))
		iMax=i;

	return iMax;
    }

    public final short computeIndexLastMaxAbs()
    {
	short iMax = 0;

	for (short i = 1; i<size; i++)
	    if (Math.abs(distribArray[i])>=Math.abs(distribArray[iMax]))
		iMax=i;

	return iMax;
    }

    public final short computeIndexMaxAbs()
    {
	short iMax = 0;
	short maxArray[] = new short[size];
	short nbMax = 1;
	maxArray[0] = 0;

	for (short i = 1; i<size; i++)
	    if (Math.abs(distribArray[i])>Math.abs(distribArray[iMax]))
		{
		    maxArray[0] = i;
		    iMax = i;
		    nbMax = 1;
		}
	    else if (Math.abs(distribArray[i])==Math.abs(distribArray[iMax]))
		{
		    maxArray[nbMax] = i;
		    nbMax++;
		}

	return maxArray[(short)(StaticRandom.nextDouble()*(double)nbMax)];
    }

    public final short computeIndexFirstMinAbs()
    {
	short iMin = 0;

	for (short i = 1; i<size; i++)
	    if (Math.abs(distribArray[i])<Math.abs(distribArray[iMin]))
		iMin=i;

	return iMin;
    }

    public final short computeIndexLastMinAbs()
    {
	short iMin = 0;

	for (short i = 1; i<size; i++)
	    if (Math.abs(distribArray[i])<=Math.abs(distribArray[iMin]))
		iMin=i;

	return iMin;
    }

    public final short computeIndexMinAbs()
    {
	short iMin = 0;
	short minArray[] = new short[size];
	short nbMin = 1;
	minArray[0] = 0;

	for (short i = 1; i<size; i++)
	    if (Math.abs(distribArray[i])<Math.abs(distribArray[iMin]))
		{
		    minArray[0] = i;
		    iMin = i;
		    nbMin = 1;
		}
	    else if (Math.abs(distribArray[i])==Math.abs(distribArray[iMin]))
		{
		    minArray[nbMin] = i;
		    nbMin++;
		}

	return minArray[(short)(StaticRandom.nextDouble()*(double)nbMin)];
    }

    public final void updateBoltzmann(double boltzmannFactor)
    {
	double sumExp = 0;

	for (short i=0; i<size; i++)
	    {
		distribArray[i] = Math.exp(boltzmannFactor*distribArray[i]);
		sumExp += distribArray[i];
	    }

	for (short i=0; i<size; i++)
	    distribArray[i] = (distribArray[i]/sumExp);
    }

    public final double computeSum()
    {
	double sum = 0;

	for (short i=0; i<size; i++)
	    sum += distribArray[i];

	return sum;
    }

    //     public final void normalize()
    //     {
    // 	normalize(0);
    //     }

    //     public final void normalize(double minValue)
    //     {
    // 	double maxValue = computeIndexFirstMax();
    // 	double enhSum = computeSum() - size*minValue;
    // 	boolean allEqual0 = true;
    // 	double oldValue = 0;

    // 	for (short i=0; i<size; i++)
    // 	    {
    // 		if (enhSum!=0)
    // 		    distribArray[i] = ((distribArray[i] - minValue) / enhSum);
    // 		if (oldValue != distribArray[i])
    // 		    allEqual0 = false;
    // 		oldValue = distribArray[i];
    // 	    }

    // 	if (allEqual0) updateToEqui();
    //     }

    private void updateToValue (double value)
    {
	for (short i=0; i<size; i++)
	    distribArray[i] = value;
    }

    private void increaseValues (double value)
    {
	for (short i=0; i<size; i++)
	    distribArray[i] += value;
    }

    private void weightValues (double value)
    {
	for (short i=0; i<size; i++)
	    distribArray[i] *= value;
    }

    public final void divideMergeWith (ValuesDistribution distribution)
    {
	for (short i=0; i<size; i++)
	    distribArray[i] = (distribArray[i] / distribution.distribArray[i]);
    }

    public final void multiplicativeMergeWith (ValuesDistribution distribution)
    {
	for (short i=0; i<size; i++)
	    distribArray[i] *= distribution.distribArray[i];
    }

    public final void additiveMergeWith (ValuesDistribution distribution)
    {
	for (short i=0; i<size; i++)
	    distribArray[i] += distribution.distribArray[i];
    }

    public final void updateToFit0_1(double minValue, double maxValue)
    {
	if (minValue==maxValue)
	    updateToValue(0.5);
	else
	    {
		increaseValues((-minValue));
		weightValues((1.0/(maxValue-minValue)));
	    }
    }

    public final void updateToFit0_1()
    {
	updateToFit0_1(distribArray[computeIndexFirstMin()], distribArray[computeIndexFirstMax()]);
    }
    
    /**
     *  Works only with values greater than 1
     **/
    public final short getRouletteWheelSelectedIndex()
    {
	double rand = (StaticRandom.nextDouble()*computeSum());
	double x=0;
	for (short i=0; i<size ; i++)
	    {
		x += distribArray[i];
		if (rand <= x)
		    return i;
	    }

	return ((short)(size-1));
    }

    public final double computeAverage()
    {
	double total = 0;

	for (short i=0; i<size; i++)
	    total += distribArray[i];

	return (total/(double)size);
    }

    public final double computeVariance()
    {
	double total = 0;
	double average = computeAverage();
	
	for (short i=0; i<size; i++)
	    total += ( (distribArray[i]-average)
			      * (distribArray[i]-average) );
	
	return (total/(double)size);
    }

    public final double computeStandardDeviation()
    {
	return Math.sqrt(computeVariance());
    }

    public final String toString()
    {
	String str = "(";
	for (short i=0 ; i<size-1 ; i++)
	    {
		str = str + String.valueOf(distribArray[i]) + " ";
	    }
	return str + String.valueOf(distribArray[size-1]) + ")";
    }

    /*
      public final static void main (String argv[])
      {
      ValuesDistribution vd1 = new ValuesDistribution((short)4);

      vd1.add(1.0);
      vd1.add(2.0);
      vd1.add(1.0);
      vd1.add(2.0);

      System.out.println ("vd1 :");
      System.out.println (vd1.toString());
      }
    */
}
